@@ -1,16 +1,18 @@
-from functools import reduce
+import enum
+from functools import reduce, singledispatch
 
 import graphene
 from django import forms
-from django.conf import settings
 from django.contrib.postgres.fields.hstore import KeyTransform
 from django.contrib.postgres.search import SearchVector
 from django.db import models
+from django.db.models import Q
 from django.db.models.constants import LOOKUP_SEP
-from django.db.models.expressions import OrderBy, RawSQL
+from django.db.models.expressions import OrderBy
 from django.db.models.fields.json import KeyTextTransform
 from django.db.models.functions import Cast
 from django.utils import translation
+from django_filters.conf import settings as filters_settings
 from django_filters.constants import EMPTY_VALUES
 from django_filters.fields import ChoiceField
 from django_filters.rest_framework import (
@@ -19,7 +21,6 @@ from django_filters.rest_framework import (
     Filter,
     FilterSet,
     MultipleChoiceFilter,
-    OrderingFilter,
 )
 from graphene import Enum, InputObjectType, List
 from graphene.types import generic
@@ -32,11 +33,7 @@ from graphene_django.forms.converter import convert_form_field
 from graphene_django.registry import get_global_registry
 from localized_fields.fields import LocalizedField
 
-from .forms import (
-    GlobalIDFormField,
-    GlobalIDMultipleChoiceField,
-    SlugMultipleChoiceField,
-)
+from .forms import GlobalIDFormField, GlobalIDMultipleChoiceField
 from .ordering import CalumaOrdering
 from .relay import extract_global_id
 from .types import DjangoConnectionField
@@ -77,6 +74,8 @@ class FilterCollectionFilter(Filter):
 
         filter_coll = self.filterset_class()
         for flt in value:
+            if not flt:
+                continue
             invert = flt.pop("invert", False)
             flt_key = list(flt.keys())[0]
             flt_val = flt[flt_key]
@@ -110,8 +109,8 @@ class FilterCollectionOrdering(Filter):
             OrderBy(
                 field,
                 descending=(direction == "DESC"),
-                nulls_first=(direction == "DESC"),
-                nulls_last=(direction == "ASC"),
+                nulls_first=(direction == "DESC" or None),
+                nulls_last=(direction == "ASC" or None),
             ),
         )
 
@@ -158,7 +157,6 @@ def FilterCollectionFactory(filterset_class, ordering):  # noqa:C901
 
     @convert_form_field.register(custom_field_class)
     def convert_field(field):
-
         registry = get_global_registry()
         converted = registry.get_converted_field(field)
         if converted:
@@ -178,8 +176,7 @@ def FilterCollectionFactory(filterset_class, ordering):  # noqa:C901
         filter_fields = {
             name: _get_or_make_field(name, filt)
             for name, filt in _filter_coll.filters.items()
-            # exclude orderBy in our fields. We want only new-style order filters
-            if _should_include_filter(filt) and name != "orderBy"
+            if _should_include_filter(filt)
         }
 
         if ordering:
@@ -203,6 +200,38 @@ def FilterCollectionFactory(filterset_class, ordering):  # noqa:C901
     return filter_coll()
 
 
+def InterfaceMetaFactory():
+    """
+    Metaclass factory for interface types.
+
+    Build a meta class suitable for the schema type classes that represent
+    "interface" types (those that have concrete subclasses, but could be mixed
+    in a connection type, such as Question, Answer, and Task).
+
+    Usage:
+
+    >>> class Foo(Node, graphene.Interface):
+    >>>     ...
+    >>>     Meta = InterfaceMetaFactory()
+    """
+
+    class _meta(graphene.types.interface.InterfaceOptions):
+        @classmethod
+        # This is kinda useless but required as graphene tries to freeze()
+        # it's meta class objects
+        def freeze(cls):
+            cls._frozen = True
+
+        # This is what we're actually "fixing": On non-Interface types,
+        # this somehow works (or isn't needed), but here, if _meta.registry
+        # is not set, the whole schema construction fails
+        registry = get_global_registry()
+
+    # We need a new type (= class) each time it's called, because reuse
+    # triggers some weird errors
+    return type("Meta", (), {"_meta": _meta})
+
+
 def CollectionFilterSetFactory(filterset_class, orderset_class=None):
     """
     Build single-filter filterset classes.
@@ -233,7 +262,7 @@ def CollectionFilterSetFactory(filterset_class, orderset_class=None):
 
     ret = CollectionFilterSetFactory._cache[cache_key] = type(
         f"{filterset_class.__name__}Collection",
-        (filterset_class, FilterSet),
+        (FilterSet,),
         {
             **coll_fields,
             "Meta": type(
@@ -241,7 +270,7 @@ def CollectionFilterSetFactory(filterset_class, orderset_class=None):
                 (filterset_class.Meta,),
                 {
                     "model": filterset_class.Meta.model,
-                    "fields": filterset_class.Meta.fields + tuple(coll_fields.keys()),
+                    "fields": tuple(coll_fields.keys()),
                 },
             ),
         },
@@ -271,16 +300,9 @@ class GlobalIDMultipleChoiceFilter(MultipleChoiceFilter):
         return super(GlobalIDMultipleChoiceFilter, self).filter(qs, gids)
 
 
-class SlugMultipleChoiceFilter(MultipleChoiceFilter):
-    field_class = SlugMultipleChoiceField
-
-    def filter(self, qs, value):
-        return super().filter(qs, value)
-
-
 class LocalizedFilter(Filter):
     def filter(self, qs, value):
-        if value in EMPTY_VALUES:
+        if value in EMPTY_VALUES:  # pragma: no cover
             return qs
 
         lang = translation.get_language()
@@ -333,7 +355,7 @@ class SearchFilter(Filter):
         return field_lookup
 
     def filter(self, qs, value):
-        if value in EMPTY_VALUES:
+        if value in EMPTY_VALUES:  # pragma: no cover
             return qs
 
         qs = qs.annotate(
@@ -345,83 +367,17 @@ class SearchFilter(Filter):
         return qs.filter(search=value)
 
 
-class OrderingField(ChoiceField):
-    """
-    Specific ordering field used as marker to convert to graphql type.
-
-    See `convert_ordering_field_to_enum`
-    """
-
-    def validate(self, value):
-        invalid = set(value or []) - {choice[0] for choice in self.choices}
-        return not bool(invalid)
-
-    def to_python(self, value):
-        return value
-
-
 class ListField(forms.Field):
     """List field as to allow actual lists in ordering vs csv string."""
 
     pass
 
 
-class OrderingFilter(OrderingFilter):
-    """Ordering filter adding default fields from models.BaseModel.
-
-    Label is required and is used for enum naming in GraphQL schema.
-
-    This filter additionally allows sorting by meta field values.
-    """
-
-    base_field_class = ListField
-    field_class = OrderingField
-
-    def __init__(self, label, *args, fields=tuple(), **kwargs):
-        fields = tuple(fields) + (
-            "created_at",
-            "modified_at",
-            "created_by_user",
-            "created_by_group",
-            "modified_by_user",
-            "modified_by_group",
-            *[f"meta_{f}" for f in settings.META_FIELDS],
-        )
-
-        super().__init__(
-            *args,
-            fields=fields,
-            label=label,
-            empty_label=None,
-            null_label=None,
-            **kwargs,
-        )
-
-    def get_ordering_value(self, param):
-        if not any(param.startswith(prefix) for prefix in ("meta_", "-meta_")):
-            return super().get_ordering_value(param)
-
-        descending = False
-        if param.startswith("-"):
-            descending = True
-            param = param[1:]
-
-        meta_field_key = param[5:]
-
-        # order_by works on json field keys only without dashes
-        # but we want to support dasherized keys as well as this is
-        # valid json, hence need to use raw sql
-        return OrderBy(
-            RawSQL(f'"{self.model._meta.db_table}"."meta"->%s', (meta_field_key,)),
-            descending=descending,
-        )
-
-
 class IntegerFilter(Filter):
     field_class = forms.IntegerField
 
 
-class FilterSet(GrapheneFilterSetMixin, FilterSet):
+class BaseFilterSet(GrapheneFilterSetMixin, FilterSet):
     created_by_user = CharFilter()
     created_by_group = CharFilter()
     modified_by_user = CharFilter()
@@ -430,12 +386,22 @@ class FilterSet(GrapheneFilterSetMixin, FilterSet):
     created_before = DateTimeFilter(
         field_name="created_at",
         lookup_expr="lt",
-        label="Only return entries created after the given DateTime (Exclusive)",
+        label="Only return entries created before the given DateTime (exclusive)",
     )
     created_after = DateTimeFilter(
         field_name="created_at",
         lookup_expr="gte",
-        label="Only return entries created at or before the given DateTime (Inclusive)",
+        label="Only return entries created at or after the given DateTime (inclusive)",
+    )
+    modified_before = DateTimeFilter(
+        field_name="modified_at",
+        lookup_expr="lt",
+        label="Only return entries modified before the given DateTime (exclusive)",
+    )
+    modified_after = DateTimeFilter(
+        field_name="modified_at",
+        lookup_expr="gte",
+        label="Only return entries modified at or after the given DateTime (inclusive)",
     )
 
 
@@ -445,6 +411,7 @@ class JSONLookupMode(Enum):
     CONTAINS = "contains"
     ICONTAINS = "icontains"
     IN = "in"
+    INTERSECTS = "intersects"
     GTE = "gte"
     GT = "gt"
     LTE = "lte"
@@ -464,15 +431,22 @@ class JSONValueFilterField(CompositeFieldClass):
 class JSONValueFilter(Filter):
     field_class = JSONValueFilterField
 
+    def __init__(self, *args, lookup_expr=None, **kwargs):
+        if lookup_expr is None:
+            lookup_expr = JSONLookupMode.get(filters_settings.DEFAULT_LOOKUP_EXPR)
+        super().__init__(*args, lookup_expr=lookup_expr, **kwargs)
+
     def filter(self, qs, value):
-        if value in EMPTY_VALUES:
+        if value in EMPTY_VALUES:  # pragma: no cover
             return qs
 
         for expr in value:
             if expr in EMPTY_VALUES:  # pragma: no cover
                 continue
 
-            lookup_expr = expr.get("lookup", self.lookup_expr)
+            lookup = expr.get("lookup") or self.lookup_expr
+            lookup_expr = (hasattr(lookup, "value") and lookup.value) or lookup
+
             # "contains" behaves differently on JSONFields as it does on TextFields.
             # That's why we annotate the queryset with the value.
             # Some discussion about it can be found here:
@@ -483,29 +457,34 @@ class JSONValueFilter(Filter):
                         KeyTextTransform(expr["key"], self.field_name),
                         models.CharField(),
                     ),
-                )
-                lookup = {f"field_val__{lookup_expr}": expr["value"]}
+                ).filter(**{f"field_val__{lookup_expr}": expr["value"]})
+            elif lookup_expr == JSONLookupMode.INTERSECTS:
+                exprs = [
+                    Q(**{f"{self.field_name}__{expr['key']}__contains": val})
+                    for val in expr["value"]
+                ]
+                # connect all expressions with OR
+                qs = qs.filter(reduce(lambda a, b: a | b, exprs))
             else:
-                lookup = {
-                    f"{self.field_name}__{expr['key']}__{lookup_expr}": expr["value"]
-                }
-            qs = qs.filter(**lookup)
+                qs = qs.filter(
+                    **{
+                        f"{self.field_name}__{expr['key']}__{lookup_expr}": expr[
+                            "value"
+                        ]
+                    }
+                )
+
         return qs
 
     @staticmethod
     @convert_form_field.register(JSONValueFilterField)
     def convert_meta_value_field(field):
-        registry = get_global_registry()
-        converted = registry.get_converted_field(field)
-        if converted:
-            return converted
-
         converted = List(JSONValueFilterType)
-        registry.register_converted_field(field, converted)
+        get_global_registry().register_converted_field(field, converted)
         return converted
 
 
-class MetaFilterSet(FilterSet):
+class MetaFilterSet(BaseFilterSet):
     meta_has_key = CharFilter(lookup_expr="has_key", field_name="meta")
     meta_value = JSONValueFilter(field_name="meta")
 
@@ -517,8 +496,62 @@ class DjangoFilterConnectionField(
     def filterset_class(self):
         return self._provided_filterset_class
 
+    @classmethod
+    def connection_resolver(
+        cls,
+        resolver,
+        connection,
+        default_manager,
+        queryset_resolver,
+        max_limit,
+        enforce_first_or_last,
+        root,
+        info,
+        **args,
+    ):
+        return super().connection_resolver(
+            resolver=resolver,
+            connection=connection,
+            default_manager=default_manager,
+            queryset_resolver=queryset_resolver,
+            max_limit=max_limit,
+            enforce_first_or_last=enforce_first_or_last,
+            root=root,
+            info=info,
+            **cls._clean_args_for_queryset_resolver(args),
+        )
+
+    @classmethod
+    def _clean_args_for_queryset_resolver(cls, args):
+        # Graphene parses incoming data into Enums too early, thus our filters
+        # will receive enum objects that cannot be parsed
+        #
+        # TODO: check if this is still required after the below
+        # resolve_queryset() is completely implemented (we assumed it's
+        # the Enums, but it was actually the list in order_by. We'll keep
+        # it here until we KNOW we can remove it again..)
+        @singledispatch
+        def clean(data):
+            return data
+
+        @clean.register(enum.Enum)
+        def _(data):
+            return data.value
+
+        @clean.register(list)
+        def _(data):
+            return [clean(e) for e in data]
+
+        @clean.register(dict)
+        def _(data):
+            return {k: clean(v) for k, v in data.items()}
+
+        return clean(args)
+
+
+class DjangoFilterInterfaceConnectionField(DjangoFilterConnectionField):
+    """Filter connection field for abstract interface types like Answer, Question and Task."""
 
-class DjangoFilterSetConnectionField(DjangoFilterConnectionField):
     @property
     def model(self):
         return self.filterset_class._meta.model
@@ -528,52 +561,6 @@ class DjangoFilterSetConnectionField(DjangoFilterConnectionField):
         return get_type(self._type)
 
 
-@convert_form_field.register(OrderingField)
-def convert_ordering_field_to_enum(field):
-    """
-    Add support to convert ordering choices to Graphql enum.
-
-    Label is used as enum name.
-    """
-    registry = get_global_registry()
-    converted = registry.get_converted_field(field)
-
-    if converted:
-        return converted
-
-    if field.label in convert_ordering_field_to_enum._cache:
-        return convert_ordering_field_to_enum._cache[field.label]
-
-    def get_choices(choices):
-        for value, help_text in choices:
-            if value[0] != "-":
-                name = convert_choice_name(value) + "_ASC"
-            else:
-                name = convert_choice_name(value[1:]) + "_DESC"
-            description = help_text
-            yield name, value, description
-
-    name = to_camel_case(field.label)
-    choices = list(get_choices(field.choices))
-    named_choices = [(c[0], c[1]) for c in choices]
-    named_choices_descriptions = {c[0]: c[2] for c in choices}
-
-    class EnumWithDescriptionsType(object):
-        @property
-        def description(self):
-            return named_choices_descriptions[self.name]
-
-    enum = Enum(name, list(named_choices), type=EnumWithDescriptionsType)
-    converted = List(enum, description=field.help_text, required=field.required)
-
-    registry.register_converted_field(field, converted)
-    convert_ordering_field_to_enum._cache[field.label] = converted
-    return converted
-
-
-convert_ordering_field_to_enum._cache = {}
-
-
 @convert_form_field.register(forms.ChoiceField)
 @convert_form_field.register(ChoiceField)
 def convert_choice_field_to_enum(field):
@@ -583,13 +570,6 @@ def convert_choice_field_to_enum(field):
     Label is used as enum name.
     """
 
-    registry = get_global_registry()
-
-    # field label of enum needs to be unique so stored it likewise
-    converted = registry.get_converted_field(field.label)
-    if converted:
-        return converted
-
     def get_choices(choices):
         for value, help_text in choices:
             if value:
@@ -610,7 +590,7 @@ def convert_choice_field_to_enum(field):
     enum = Enum(name, list(named_choices), type=EnumWithDescriptionsType)
     converted = enum(description=field.help_text, required=field.required)
 
-    registry.register_converted_field(field.label, converted)
+    get_global_registry().register_converted_field(field.label, converted)
     return converted
 
 
@@ -638,9 +618,11 @@ def generate_list_filter_class(inner_type):
             ),
         },
     )
-    convert_form_field.register(form_field)(
-        lambda x: graphene.List(inner_type, required=x.required)
-    )
+
+    def do_convert_type(x):
+        return graphene.List(inner_type, required=x.required)
+
+    convert_form_field.register(form_field)(do_convert_type)
 
     return filter_class
 
