@@ -2,7 +2,7 @@ import json
 import math
 import numbers
 from functools import partial
-from itertools import count
+from itertools import chain, count
 from logging import getLogger
 
 import pyjexl
@@ -74,6 +74,8 @@ class JEXL(pyjexl.JEXL):
 
         self.add_transform("mapby", self._mapby_transform)
         self.add_transform("stringify", lambda obj: json.dumps(obj))
+        self.add_transform("flatten", self._flatten_transform)
+        self.add_transform("length", self._length_transform)
         self.add_binary_operator(
             "intersects", 20, lambda left, right: any(x in right for x in left)
         )
@@ -117,7 +119,7 @@ class JEXL(pyjexl.JEXL):
         )
 
     def _round_compat(self, num, ndigits=0):
-        power = 10 ** ndigits
+        power = 10**ndigits
         return float(math.floor((num * power) + 0.5) / power)
 
     def parse(self, expression):
@@ -152,6 +154,18 @@ class JEXL(pyjexl.JEXL):
             for obj in arr
         ]
 
+    def _flatten_transform(self, arr, *options):
+        if not isinstance(arr, list):
+            return None
+
+        return list(chain(*arr))
+
+    def _length_transform(self, value, *options):
+        try:
+            return len(value)
+        except TypeError:
+            return None
+
     def evaluate(self, expression, context=None):
         self._expr_stack.append(expression)
         try:
@@ -229,18 +243,3 @@ class ExtractTransformArgumentAnalyzer(CalumaAnalyzer):
                         yield arg.value
 
         yield from self.generic_visit(transform)
-
-
-class ExtractTransformSubjectAndArgumentsAnalyzer(CalumaAnalyzer):
-    """
-    Extract all referenced subjects and arguments of a given transforms.
-
-    If no transforms are given all references of all transforms will be extracted.
-    """
-
-    def visit_Transform(self, transform):
-        if not self.transforms or transform.name in self.transforms:
-            if not isinstance(transform.subject, type(transform)):
-                yield (transform.subject.value, transform.args)
-
-        yield from self.generic_visit(transform)
