@@ -26,22 +26,22 @@ class ASTAnalyzer:
         def _parse_ast_value(arg, info):  # pragma: no cover
             raise RuntimeError(f"Unknown arg {arg}")
 
-        @_parse_ast_value.register(ast.ObjectValue)
+        @_parse_ast_value.register(ast.ObjectValueNode)
         def _(arg, info):
             return {
                 field.name.value: _parse_ast_value(field.value, info)
                 for field in arg.fields
             }
 
-        @_parse_ast_value.register(ast.ListValue)
+        @_parse_ast_value.register(ast.ListValueNode)
         def _(arg, info):
             return [_parse_ast_value(val, info) for val in arg.values]
 
-        @_parse_ast_value.register(ast.StringValue)
+        @_parse_ast_value.register(ast.StringValueNode)
         def _(arg, info):
             return arg.value
 
-        @_parse_ast_value.register(ast.Variable)
+        @_parse_ast_value.register(ast.VariableNode)
         def _(arg, info):
             return info.variable_values[arg.name.value]
 
@@ -54,7 +54,7 @@ class ASTAnalyzer:
             for sel in info.operation.selection_set.selections
             if sel.name.value == info.field_name
             # if aliases are used, we need to compare them as well
-            and (sel.alias is None or sel.alias.value == info.path[0])
+            and (sel.alias is None or sel.alias.value == info.path.key)
         ][0]
 
         return {
@@ -119,7 +119,7 @@ class Mutation(ClientIDMutation):
     permission_classes = None
 
     @classmethod
-    def __init_subclass_with_meta__(
+    def __init_subclass_with_meta__(  # noqa: C901
         cls,
         lookup_field=None,
         lookup_input_kwarg=None,
@@ -154,6 +154,17 @@ class Mutation(ClientIDMutation):
 
         input_fields = fields_for_serializer(serializer, fields, exclude, is_input=True)
 
+        if hasattr(cls, "Input"):
+            # Allow input type override in case the serializer fields cannot
+            # define exactly what we need
+            input_obj = cls.Input()
+            explicit_inputs = [
+                name for name in dir(input_obj) if not name.startswith("_")
+            ]
+            for name in explicit_inputs:
+                input_type = getattr(input_obj, name)
+                input_fields[name] = input_type
+
         if return_field_name is None:
             model_name = model_class.__name__
             return_field_name = model_name[:1].lower() + model_name[1:]
@@ -259,7 +270,6 @@ class Mutation(ClientIDMutation):
 
     @classmethod
     def perform_mutate(cls, serializer, info):
-
         obj = serializer.save()
         kwargs = {}
         if cls._meta.return_field_name:
