@@ -1,6 +1,7 @@
 import logging
+from typing import Tuple
 
-from caluma.caluma_form.models import DynamicOption
+from caluma.caluma_form.models import Answer, DynamicOption
 from caluma.utils import is_iterable_and_no_string
 
 logger = logging.getLogger(__name__)
@@ -14,9 +15,10 @@ class BaseDataSource:
     two items. The first will be used for the option name, the second one for it's
     value. If only one value is provided, this value will also be used as choice name.
 
-    The `validate_answer_value`-method checks if each value in `self.get_data(user)` equals the value
-    of the parameter `value`. If this is correct the method returns the label as a String
-    and otherwise the method returns `False`.
+    The `validate_answer_value`-method checks if each value in
+    `self.get_data(user, question, context)` equals the value of the parameter
+    `value`. If this is correct the method returns the label as a String and
+    otherwise the method returns `False`.
 
     Examples:
         [['my-option', {"en": "english description", "de": "deutsche Beschreibung"}, ...]
@@ -40,7 +42,7 @@ class BaseDataSource:
     ...     info = 'User choices from "someapi"'
     ...
     ...     @data_source_cache(timeout=3600)
-    ...     def get_data(self, user):
+    ...     def get_data(self, user, question, context):
     ...         response = requests.get(f"https://someapi/?user={user.username}")
     ...         return [result["value"] for result in response.json()["results"]]
     ```
@@ -53,11 +55,11 @@ class BaseDataSource:
     def __init__(self):
         pass
 
-    def get_data(self, user):  # pragma: no cover
+    def get_data(self, user, question, context):  # pragma: no cover
         raise NotImplementedError()
 
-    def validate_answer_value(self, value, document, question, user):
-        for data in self.get_data(user):
+    def validate_answer_value(self, value, document, question, user, context):
+        for data in self.get_data(user, question, context):
             label = data
             if is_iterable_and_no_string(data):
                 label = data[-1]
@@ -73,9 +75,9 @@ class BaseDataSource:
             return dynamic_option.label
         return False
 
-    def try_get_data_with_fallback(self, user):
+    def try_get_data_with_fallback(self, user, question, context):
         try:
-            new_data = self.get_data(user)
+            new_data = self.get_data(user, question, context)
         except Exception as e:
             logger.exception(
                 f"Executing {type(self).__name__}.get_data() failed:"
@@ -85,3 +87,25 @@ class BaseDataSource:
                 raise e
             return self.default
         return new_data
+
+    def on_copy(
+        self, old_answer: Answer, new_answer: Answer, old_value: Tuple[str, str]
+    ) -> Tuple[str | None, str | None]:
+        """Alter the dynamic option when an Answer with this datasource gets copied.
+
+        When an answer of type TYPE_DYNAMIC_CHOICE or TYPE_DYNAMIC_MULTIPLE_CHOICE
+        gets copied the meaning of the slug and label of the dynamic option could
+        potentially change when the datasource data changes. During the answer copy
+        process the linked datasource it's on_copy method will be called to decide what
+        to do with existing answer values.
+
+        The decided outcome can be either retain, change or discard as follows:
+
+        - return the same slug,label tuple to not perform any change (default behavior)
+        - return an altered slug or(/and) label to save the answer value with
+            a new value or(/and) update the dynamic option label
+        - return a None value in the tuple for the slug, which will discard the
+            answer value, and prevent copying the dynamic option
+
+        """
+        return old_value
