@@ -1,5 +1,14 @@
 import faker
-from factory import Faker, LazyAttribute, Maybe, SubFactory, lazy_attribute
+from django.db.models.signals import post_save
+from factory import (
+    Faker,
+    LazyAttribute,
+    Maybe,
+    SubFactory,
+    django,
+    lazy_attribute,
+    post_generation,
+)
 
 from ..caluma_core.factories import DjangoModelFactory
 from . import models
@@ -81,6 +90,11 @@ class QuestionFactory(DjangoModelFactory):
         yes_declaration=False,
         no_declaration=None,
     )
+    show_validation = Maybe(
+        "is_action_button",
+        yes_declaration=False,
+        no_declaration=None,
+    )
 
     class Meta:
         model = models.Question
@@ -107,6 +121,7 @@ class QuestionFactory(DjangoModelFactory):
 class OptionFactory(DjangoModelFactory):
     slug = Faker("slug")
     label = Faker("multilang", faker_provider="name")
+    is_hidden = "false"
     is_archived = False
     meta = {}
 
@@ -116,8 +131,8 @@ class OptionFactory(DjangoModelFactory):
 
 class QuestionOptionFactory(DjangoModelFactory):
     option = SubFactory(OptionFactory)
-    question = SubFactory(QuestionFactory)
-    sort = 0
+    question = SubFactory(QuestionFactory, type=models.Question.TYPE_CHOICE)
+    sort = Faker("pyint")
 
     class Meta:
         model = models.QuestionOption
@@ -141,13 +156,6 @@ class DocumentFactory(DjangoModelFactory):
         model = models.Document
 
 
-class FileFactory(DjangoModelFactory):
-    name = Faker("file_name")
-
-    class Meta:
-        model = models.File
-
-
 class AnswerFactory(DjangoModelFactory):
     question = SubFactory(QuestionFactory)
     document = SubFactory(DocumentFactory)
@@ -166,26 +174,46 @@ class AnswerFactory(DjangoModelFactory):
             return faker.Faker().pyint()
         elif self.question.type not in [
             models.Question.TYPE_TABLE,
-            models.Question.TYPE_FILE,
+            models.Question.TYPE_FILES,
             models.Question.TYPE_DATE,
         ]:
             return faker.Faker().name()
 
         return None
 
-    file = Maybe(
-        "is_file", yes_declaration=SubFactory(FileFactory), no_declaration=None
-    )
     date = Maybe("is_date", yes_declaration=Faker("date"), no_declaration=None)
 
+    @post_generation
+    @django.mute_signals(post_save)
+    def files(self, create, extracted, **kwargs):
+        if not create:  # pragma: no cover
+            return
+        if self.question.type == models.Question.TYPE_FILES:
+            if extracted is not None:
+                self.files.set(extracted)
+            else:
+                num_files = kwargs.pop("count", 1)
+                self.files.set(
+                    FileFactory.create_batch(num_files, **kwargs, answer=self)
+                )
+            self.save()
+
     class Meta:
         model = models.Answer
+        skip_postgeneration_save = True
 
     class Params:
-        is_file = LazyAttribute(lambda a: a.question.type == models.Question.TYPE_FILE)
         is_date = LazyAttribute(lambda a: a.question.type == models.Question.TYPE_DATE)
 
 
+class FileFactory(DjangoModelFactory):
+    name = Faker("file_name")
+    answer = SubFactory(AnswerFactory)
+
+    class Meta:
+        model = models.File
+
+
 class AnswerDocumentFactory(DjangoModelFactory):
     answer = SubFactory(AnswerFactory)
     document = SubFactory(DocumentFactory)
