@@ -1,6 +1,6 @@
-import uuid
 from functools import wraps
 
+import uuid_extensions
 from django.contrib.postgres.fields import ArrayField
 from django.contrib.postgres.indexes import GinIndex
 from django.db import models, transaction
@@ -9,6 +9,8 @@ from localized_fields.fields import LocalizedField, LocalizedTextField
 from minio import S3Error
 from simple_history.models import HistoricalRecords
 
+from caluma.caluma_data_source.data_source_handlers import get_data_sources
+
 from ..caluma_core import models as core_models
 from .storage_clients import client
 
@@ -31,6 +33,41 @@ class Form(core_models.SlugModel):
         on_delete=models.SET_NULL,
     )
 
+    @classmethod
+    def _get_all_raw_forms(cls, forms: list):
+        return cls.objects.raw(
+            """
+            WITH RECURSIVE
+                forms(slug) AS (
+                    SELECT slug
+                    FROM caluma_form_form
+                    WHERE slug = ANY (%s)
+                    UNION ALL
+                    SELECT
+                        all_subforms.subform_id AS slug
+                    FROM forms JOIN all_subforms ON (all_subforms.form_id = forms.slug )
+                ),
+                all_subforms(form_id, subform_id) AS (
+                    SELECT
+                        caluma_form_formquestion.form_id,
+                        COALESCE(caluma_form_question.row_form_id, caluma_form_question.sub_form_id) AS subform_id
+                    FROM caluma_form_formquestion
+                    JOIN caluma_form_question ON (caluma_form_formquestion.question_id = caluma_form_question.slug)
+                    WHERE caluma_form_question.type IN ('form', 'table')
+                )
+            SELECT * FROM forms;
+        """,
+            [forms],
+        )
+
+    @classmethod
+    def get_all_forms(cls, forms: list):  # pragma: no cover
+        return Form.objects.filter(pk__in=cls._get_all_raw_forms(forms))
+
+    @classmethod
+    def get_all_questions(cls, forms: list):
+        return Question.objects.filter(forms__in=cls._get_all_raw_forms(forms))
+
     class Meta:
         indexes = [GinIndex(fields=["meta"])]
 
@@ -58,7 +95,7 @@ class Question(core_models.SlugModel):
     TYPE_TEXT = "text"
     TYPE_TABLE = "table"
     TYPE_FORM = "form"
-    TYPE_FILE = "file"
+    TYPE_FILES = "files"
     TYPE_DYNAMIC_CHOICE = "dynamic_choice"
     TYPE_DYNAMIC_MULTIPLE_CHOICE = "dynamic_multiple_choice"
     TYPE_STATIC = "static"
@@ -75,7 +112,7 @@ class Question(core_models.SlugModel):
         TYPE_TEXT,
         TYPE_TABLE,
         TYPE_FORM,
-        TYPE_FILE,
+        TYPE_FILES,
         TYPE_DYNAMIC_CHOICE,
         TYPE_DYNAMIC_MULTIPLE_CHOICE,
         TYPE_STATIC,
@@ -201,6 +238,14 @@ class Question(core_models.SlugModel):
     def color(self, value):
         self.configuration["color"] = value
 
+    @property
+    def step(self):
+        return self.configuration.get("step")
+
+    @step.setter
+    def step(self, value):
+        self.configuration["step"] = value
+
     @property
     def validate_on_enter(self):
         return self.configuration.get("validate_on_enter")
@@ -209,6 +254,14 @@ class Question(core_models.SlugModel):
     def validate_on_enter(self, value):
         self.configuration["validate_on_enter"] = value
 
+    @property
+    def show_validation(self):
+        return self.configuration.get("show_validation")
+
+    @show_validation.setter
+    def show_validation(self, value):
+        self.configuration["show_validation"] = value
+
     def empty_value(self):
         """Return empty value for this question type."""
 
@@ -242,6 +295,7 @@ class QuestionOption(core_models.NaturalKeyModel):
 
 class Option(core_models.SlugModel):
     label = LocalizedField(blank=False, null=False, required=False)
+    is_hidden = models.TextField(default="false")
     is_archived = models.BooleanField(default=False)
     meta = models.JSONField(default=dict)
     source = models.ForeignKey(
@@ -291,6 +345,40 @@ class Document(core_models.UUIDModel):
     )
     meta = models.JSONField(default=dict)
 
+    def flat_answer_map(self):
+        """
+        Return a dictionary with the flattened answer map for this document.
+
+        The keys are the question IDs, and the values are either the answer value
+        or date for non-table questions, or a list of flattened answer maps for
+        table questions.
+
+        Example usage:
+        >>> doc = Document.objects.get(pk=1)
+        >>> flat_map = doc.flat_answer_map()
+        >>> print(flat_map)
+        {
+            "first-name": 'John',
+            "second-name": 'Doe',
+            "email-in-subform": 'john.doe@example.com',
+            "projects": [
+                {"title": 'Project A', "date": '2022-01-01'},
+                {"title": 'Project B', "date": '2022-02-01'},
+                {"title": 'Project C', "date": '2022-03-01'}
+            ]
+        }
+        """
+        answers = {}
+        for answer in self.answers.all():
+            if answer.question.type == Question.TYPE_TABLE:
+                answers[answer.question_id] = [
+                    answer.flat_answer_map() for answer in answer.documents.all()
+                ]
+            else:
+                answers[answer.question_id] = answer.value or answer.date
+
+        return answers
+
     def set_family(self, root_doc):
         """Set the family to the given root_doc.
 
@@ -319,9 +407,14 @@ class Document(core_models.UUIDModel):
     def copy(self, family=None, user=None):
         """Create a copy including all its answers."""
 
+        # no need to update calculated questions, since calculated answers
+        # are copied as well
+        meta = dict(self.meta)
+        meta["_defer_calculation"] = True
+
         new_document = type(self).objects.create(
             form=self.form,
-            meta=dict(self.meta),
+            meta=meta,
             source=self,
             family=family,
             created_by_user=user.username if user else None,
@@ -329,17 +422,95 @@ class Document(core_models.UUIDModel):
             modified_by_user=user.username if user else None,
             modified_by_group=user.group if user else None,
         )
+
+        is_main_doc = False
         if not family:
+            is_main_doc = True
             family = new_document
 
-        # copy answers
         for source_answer in self.answers.all():
             source_answer.copy(
                 document_family=family, to_document=new_document, user=user
             )
 
+        new_document.meta.pop("_defer_calculation", None)
+        new_document.save()
+
+        if is_main_doc:
+            self._update_dynamic_answers_after_copy(family)
+
         return new_document
 
+    def _update_dynamic_answers_after_copy(self, family):
+        """Update all dynamic answers of the document family.
+
+        when the main doc is completely copied, this means all nested documents and
+        answers have been copied as well. Fetch all answers of the document family
+        and update the dynamic options of the answers through the data source on_copy
+        method.
+        """
+
+        # get all dynamic answers of the document family
+        family_dynamic_answers = Answer.objects.filter(
+            document__in=Document.objects.filter(family=family).values_list(
+                "pk", flat=True
+            ),
+            question__type__in=[
+                Question.TYPE_DYNAMIC_CHOICE,
+                Question.TYPE_DYNAMIC_MULTIPLE_CHOICE,
+            ],
+        )
+
+        for new_answer in family_dynamic_answers:
+            answer_value_changed = False
+
+            # initialize the data source used by the dynamic question
+            data_source_class = get_data_sources(dic=True)[
+                new_answer.question.data_source
+            ]
+            data_source = data_source_class()
+
+            for new_dynamic_option in DynamicOption.objects.filter(
+                document=new_answer.document, question=new_answer.question
+            ):
+                # get the old answer from the source document for comparison
+                old_answer = Answer.objects.get(
+                    document=new_answer.document.source,
+                    question=new_answer.question,
+                )
+
+                # let the data source decide what to do with the answer value.
+                new_slug, new_label = data_source.on_copy(
+                    old_answer=old_answer,
+                    new_answer=new_answer,
+                    old_value=(new_dynamic_option.slug, new_dynamic_option.label),
+                )
+
+                # modify the answer value if the datasource decides to change or
+                # discard the answer value.
+                if new_slug is None or new_slug != new_dynamic_option.slug:
+                    answer_value_changed = True
+                    new_answer.value = new_answer.modify_changed_choice_answer(
+                        new_answer.question,
+                        new_dynamic_option.slug,
+                        new_slug,
+                        new_answer.value,
+                    )
+
+                # update the dynamic option if the slug has changed
+                if new_slug and new_slug != new_dynamic_option.slug:
+                    new_dynamic_option.slug = new_slug
+                    new_dynamic_option.label = new_label
+                    new_dynamic_option.save()
+
+                # delete the dynamic option if the slug is None
+                elif not new_slug:
+                    new_dynamic_option.delete()
+
+            # save the answer value if changed after checking all dynamic options
+            if answer_value_changed:
+                new_answer.save(update_fields=["value"])
+
     @cached_property
     def last_modified_answer(self):
         """Get most recently modified answer of the document.
@@ -389,7 +560,9 @@ class Answer(core_models.BaseModel):
 
     # We need to replicate the UUIDModel in order to register it as historical model.
     # Otherwise simple_history complains that the model is already registered,
-    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
+    id = models.UUIDField(
+        primary_key=True, default=uuid_extensions.uuid7, editable=False
+    )
     question = models.ForeignKey(
         "caluma_form.Question", on_delete=models.DO_NOTHING, related_name="answers"
     )
@@ -406,9 +579,6 @@ class Answer(core_models.BaseModel):
         Document, through="AnswerDocument", related_name="+"
     )
     date = models.DateField(null=True, blank=True)
-    file = models.OneToOneField(
-        "File", on_delete=models.SET_NULL, null=True, blank=True
-    )
 
     # override history to add extra fields on historical model
     history = HistoricalRecords(
@@ -420,13 +590,21 @@ class Answer(core_models.BaseModel):
     )
 
     def delete(self, *args, **kwargs):
-        if self.file:
-            self.file.delete()
+        # Files need to be deleted in sequence (not via on_delete)
+        # so the deletion code on the storage is properly triggered
+        for file in self.files.all():
+            file.delete()
         super().delete(args, kwargs)
 
     def create_answer_documents(self, documents):
+        """Create AnswerDocuments for this table question, and attach them.
+
+        Return a dict with two keys: "created", and "updated", each
+        containing a list of document IDs that were either created or kept.
+        """
         family = getattr(self.document, "family", None)
         document_ids = [document.pk for document in documents]
+        res = {"updated": [], "created": []}
 
         for sort, document_id in enumerate(reversed(document_ids), start=1):
             ans_doc, created = AnswerDocument.objects.get_or_create(
@@ -439,6 +617,10 @@ class Answer(core_models.BaseModel):
                 # Already-existing documents are already in the family,
                 # so we're updating only the newly attached rows
                 ans_doc.document.set_family(family)
+                res["created"].append(document_id)
+            else:
+                res["updated"].append(document_id)
+        return res
 
     def unlink_unused_rows(self, docs_to_keep):
         existing = AnswerDocument.objects.filter(answer=self).exclude(
@@ -450,22 +632,73 @@ class Answer(core_models.BaseModel):
             ans_doc.document.set_family(ans_doc.document)
             ans_doc.delete()
 
+    def modify_changed_choice_answer(self, question, old_slug, new_slug, answer_value):
+        """Handle answer value updates based on the datasource on_copy decision.
+
+        For single dynamic choice questions:
+        - The value will be changed to None for discard operations (None will be the
+            value of new_slug)
+        - The value will be changed to the new slug for change operations
+        - The value will remain unchanged for retain operations
+
+        For multiple dynamic choice questions:
+        - The slug will be removed from the list for discard operations
+        - The slug will be replaced in the list with the new slug for change operations
+        - The list will remain unchanged for retain operations
+
+        """
+        if question.type == Question.TYPE_DYNAMIC_CHOICE:
+            return new_slug if answer_value == old_slug else answer_value
+
+        if question.type == Question.TYPE_DYNAMIC_MULTIPLE_CHOICE:
+            # don't modify the answer if it has no value
+            if not answer_value:
+                return answer_value
+
+            if old_slug in answer_value:
+                if new_slug is None:
+                    answer_value.remove(old_slug)
+                else:
+                    answer_value[answer_value.index(old_slug)] = new_slug
+
+        return answer_value
+
     def copy(self, document_family=None, to_document=None, user=None):
-        new_answer = type(self).objects.create(
+        new_answer, _ = type(self).objects.update_or_create(
             question=self.question,
-            value=self.value,
-            meta=dict(self.meta),
-            date=self.date,
             document=to_document,
-            created_by_user=user.username if user else None,
-            created_by_group=user.group if user else None,
-            modified_by_user=user.username if user else None,
-            modified_by_group=user.group if user else None,
+            defaults={
+                "value": self.value,
+                "meta": dict(self.meta),
+                "date": self.date,
+                "created_by_user": user.username if user else None,
+                "created_by_group": user.group if user else None,
+                "modified_by_user": user.username if user else None,
+                "modified_by_group": user.group if user else None,
+            },
         )
 
-        if self.question.type == Question.TYPE_FILE:
-            new_answer.file = self.file.copy()
-            new_answer.save()
+        if self.question.type == Question.TYPE_FILES:
+            for file in self.files.all():
+                file.copy(to_answer=new_answer)
+
+        if self.question.type in [
+            Question.TYPE_DYNAMIC_CHOICE,
+            Question.TYPE_DYNAMIC_MULTIPLE_CHOICE,
+        ]:
+            for dynamic_option in DynamicOption.objects.filter(
+                document=self.document, question=self.question
+            ):
+                DynamicOption.objects.update_or_create(
+                    document=to_document,
+                    question=dynamic_option.question,
+                    slug=dynamic_option.slug,
+                    defaults={
+                        "label": dynamic_option.label,
+                        "created_by_user": user.username if user else None,
+                        "created_by_group": user.group if user else None,
+                    },
+                )
 
         # TableAnswer: copy AnswerDocument too
         for answer_doc in AnswerDocument.objects.filter(answer=self):
@@ -505,11 +738,20 @@ class Answer(core_models.BaseModel):
             ),
         }
 
-        if not self.value or self.question.type not in map:
+        if self.question.type not in map:
             return None
 
+        if not self.value:
+            return []
+
         model, filters = map[self.question.type]
-        return model.objects.filter(**filters)
+        queryset = model.objects.filter(**filters)
+
+        def index_from_value(obj):
+            return self.value.index(obj.slug)
+
+        # Keep the same order as the value
+        return sorted(queryset, key=index_from_value)
 
     def __repr__(self):
         return f"Answer(document={self.document!r}, question={self.question!r}, value={self.value!r})"
@@ -542,11 +784,18 @@ def _ignore_missing_file(fn):
 class File(core_models.UUIDModel):
     name = models.CharField(max_length=255)
 
+    answer = models.ForeignKey(
+        Answer, on_delete=models.CASCADE, related_name="files", null=True, default=None
+    )
+
     @_ignore_missing_file
     def _move_blob(self):
         # move the file on update
         # this makes sure it stays available when querying the history
         old_file = self.history.first()
+        if not old_file:  # pragma: no cover
+            # No history - cannot move object to preserve history.
+            return
         new_name = f"{old_file.pk}_{old_file.name}"
         client.move_object(self.object_name, new_name)
 
@@ -554,11 +803,16 @@ class File(core_models.UUIDModel):
     def _copy(self, new_object_name):
         client.copy_object(self.object_name, new_object_name)
 
-    def copy(self):
-        copy = File.objects.create(name=self.name)
+    def copy(self, to_answer):
+        copy = File.objects.create(name=self.name, answer=to_answer or self.answer)
         self._copy(copy.object_name)
         return copy
 
+    def rename(self, new_name):
+        from_object_name = self.object_name
+        self.name = new_name
+        client.move_object(from_object_name, self.object_name)
+
     def delete(self, *args, **kwargs):
         self._move_blob()
         super().delete(*args, **kwargs)
