@@ -4,10 +4,13 @@ from graphene import relay
 from graphene.types import ObjectType, generic
 from graphene_django.rest_framework import serializer_converter
 
+from caluma.utils import suppressable_visibility_resolver
+
 from ..caluma_core.filters import (
     CollectionFilterSetFactory,
     DjangoFilterConnectionField,
-    DjangoFilterSetConnectionField,
+    DjangoFilterInterfaceConnectionField,
+    InterfaceMetaFactory,
 )
 from ..caluma_core.mutation import Mutation, UserDefinedPrimaryKeyMixin
 from ..caluma_core.relay import extract_global_id
@@ -16,6 +19,7 @@ from ..caluma_core.types import (
     CountableConnectionBase,
     DjangoObjectType,
     Node,
+    enum_type_from_field,
 )
 from ..caluma_data_source.data_source_handlers import get_data_source_data
 from ..caluma_data_source.schema import DataSourceDataConnection
@@ -63,19 +67,38 @@ class FormDjangoObjectType(DjangoObjectType):
 class QuestionJexl(graphene.String):
     """Question jexl expression returning boolean.
 
-    Following transform can be used:
-    * answer - access answer of document by question slug
-    * mapby - map list by key. Helpful to work with table answers
+    Following transforms can be used:
+    * `answer`: get answer of document by question slug
+    * `mapby`: map list by key. Helpful to work with table answers
       whereas an answer is a list of dicts.
+    * `stringify`: JSON stringify
+    * `flatten`: flatten list values
+    * `min`: get min value in a list
+    * `max`: get max value in a list
+    * `sum`: sum of a list
+    * `round`: round the value
+    * `ceil`: round value up
+    * `floor`: round value down
+    * `debug`: debug output
+
+    Following binary operators can be used:
+    * `intersects`: list intersection operator
 
     Following context is available:
-    * form - access form of document
+    * `form`: legacy property pointing to the root form (this should not be used anymore)
+    * `info.form`: slug of the form this question is attached to
+    * `info.formMeta`: meta property of the form this question is attached to
+    * `info.parent.form`: parent form slug
+    * `info.parent.formMeta`: parent form meta property
+    * `info.root.form`: top level form slug
+    * `info.root.formMeta`: top level form meta property
 
     Examples:
     * 'answer' == 'question-slug'|answer
     * 'answer' in 'list-question-slug'|answer
     * 'answer' in 'table-question-slug'|answer|mapby('column-question')
-    * 'form-slug' == form
+    * 'answer' in 'table-question-slug'|answer|mapby('multiple-choice-question')|flatten
+    * 'form-slug' == info.form
 
     """
 
@@ -87,22 +110,16 @@ serializer_converter.get_graphene_type_from_serializer_field.register(
 )
 
 
-ButtonAction = graphene.Enum(
+ButtonAction = enum_type_from_field(
     "ButtonAction",
-    [(key.upper(), key) for key in models.Question.ACTION_CHOICES],
+    choices=models.Question.ACTION_CHOICES,
+    serializer_field=serializers.ButtonActionField,
 )
-ButtonColor = graphene.Enum(
+
+ButtonColor = enum_type_from_field(
     "ButtonColor",
-    [(key.upper(), key) for key in models.Question.COLOR_CHOICES],
-)
-
-
-serializer_converter.get_graphene_type_from_serializer_field.register(
-    serializers.ButtonActionField, lambda field: ButtonAction
-)
-
-serializer_converter.get_graphene_type_from_serializer_field.register(
-    serializers.ButtonColorField, lambda field: ButtonColor
+    choices=models.Question.COLOR_CHOICES,
+    serializer_field=serializers.ButtonColorField,
 )
 
 
@@ -125,10 +142,15 @@ class Question(Node, graphene.Interface):
     is_archived = graphene.Boolean(required=True)
     meta = generic.GenericScalar(required=True)
     forms = DjangoFilterConnectionField(
-        "caluma.caluma_form.schema.Form", filterset_class=filters.FormFilterSet
+        "caluma.caluma_form.schema.Form",
+        filterset_class=CollectionFilterSetFactory(
+            filterset_class=filters.FormFilterSet, orderset_class=filters.FormOrderSet
+        ),
     )
     source = graphene.Field("caluma.caluma_form.schema.Question")
 
+    resolve_source = suppressable_visibility_resolver()
+
     @classmethod
     def get_queryset(cls, queryset, info):
         queryset = super().get_queryset(queryset, info)
@@ -142,10 +164,14 @@ class Question(Node, graphene.Interface):
     def resolve_type(cls, instance, info):
         return resolve_question(instance)
 
+    Meta = InterfaceMetaFactory()
+
 
 class Option(FormDjangoObjectType):
     meta = generic.GenericScalar()
 
+    resolve_source = suppressable_visibility_resolver()
+
     class Meta:
         model = models.Option
         interfaces = (relay.Node,)
@@ -191,7 +217,7 @@ class TextQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
     format_validators = ConnectionField(FormatValidatorConnection)
     default_answer = graphene.Field("caluma.caluma_form.schema.StringAnswer")
 
-    def resolve_format_validators(self, info):
+    def resolve_format_validators(self, info, *args, **kwargs):
         return get_format_validators(include=self.format_validators)
 
     class Meta:
@@ -221,7 +247,7 @@ class TextareaQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
     format_validators = ConnectionField(FormatValidatorConnection)
     default_answer = graphene.Field("caluma.caluma_form.schema.StringAnswer")
 
-    def resolve_format_validators(self, info):
+    def resolve_format_validators(self, info, *args, **kwargs):
         return get_format_validators(include=self.format_validators)
 
     class Meta:
@@ -270,7 +296,11 @@ class DateQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
 
 class ChoiceQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
     options = DjangoFilterConnectionField(
-        Option, filterset_class=filters.OptionFilterSet
+        Option,
+        filterset_class=CollectionFilterSetFactory(
+            filterset_class=filters.OptionFilterSet,
+            orderset_class=filters.OptionOrderSet,
+        ),
     )
     hint_text = graphene.String()
     default_answer = graphene.Field("caluma.caluma_form.schema.StringAnswer")
@@ -297,7 +327,11 @@ class ChoiceQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
 
 class MultipleChoiceQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
     options = DjangoFilterConnectionField(
-        Option, filterset_class=filters.OptionFilterSet
+        Option,
+        filterset_class=CollectionFilterSetFactory(
+            filterset_class=filters.OptionFilterSet,
+            orderset_class=filters.OptionOrderSet,
+        ),
     )
     hint_text = graphene.String()
     default_answer = graphene.Field("caluma.caluma_form.schema.ListAnswer")
@@ -322,12 +356,17 @@ class MultipleChoiceQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
 
 
 class DynamicQuestion(graphene.Interface):
-    options = ConnectionField(DataSourceDataConnection)
+    options = ConnectionField(
+        DataSourceDataConnection,
+        context=graphene.JSONString(
+            description="JSON object passed as context to the data source"
+        ),
+    )
     data_source = graphene.String(required=True)
     hint_text = graphene.String()
 
-    def resolve_options(self, info, *args):
-        return get_data_source_data(info.context.user, self.data_source)
+    def resolve_options(self, info, context=None, *args, **kwargs):
+        return get_data_source_data(info.context.user, self.data_source, self, context)
 
 
 class DynamicChoiceQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
@@ -385,6 +424,7 @@ class DynamicOption(DjangoObjectType):
         model = models.DynamicOption
         interfaces = (relay.Node,)
         connection_class = CountableConnectionBase
+        fields = "__all__"
 
 
 class IntegerQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
@@ -417,6 +457,7 @@ class IntegerQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
 class FloatQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
     min_value = graphene.Float()
     max_value = graphene.Float()
+    step = graphene.Float()
     placeholder = graphene.String()
     hint_text = graphene.String()
     default_answer = graphene.Field("caluma.caluma_form.schema.FloatAnswer")
@@ -488,7 +529,7 @@ class FormQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
         interfaces = (Question, graphene.Node)
 
 
-class FileQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
+class FilesQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
     hint_text = graphene.String()
 
     class Meta:
@@ -569,6 +610,7 @@ class ActionButtonQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
     action = ButtonAction(required=True)
     color = ButtonColor(required=True)
     validate_on_enter = graphene.Boolean(required=True)
+    show_validation = graphene.Boolean(required=True)
 
     class Meta:
         model = models.Question
@@ -594,11 +636,17 @@ class ActionButtonQuestion(QuestionQuerysetMixin, FormDjangoObjectType):
 
 
 class Form(FormDjangoObjectType):
-    questions = DjangoFilterSetConnectionField(
-        QuestionConnection, filterset_class=filters.QuestionFilterSet
+    questions = DjangoFilterInterfaceConnectionField(
+        QuestionConnection,
+        filterset_class=CollectionFilterSetFactory(
+            filterset_class=filters.QuestionFilterSet,
+            orderset_class=filters.QuestionOrderSet,
+        ),
     )
     meta = generic.GenericScalar()
 
+    resolve_source = suppressable_visibility_resolver()
+
     class Meta:
         model = models.Form
         interfaces = (relay.Node,)
@@ -724,7 +772,7 @@ class SaveFormQuestion(SaveQuestion):
         return_field_type = Question
 
 
-class SaveFileQuestion(SaveQuestion):
+class SaveFilesQuestion(SaveQuestion):
     class Meta:
         serializer_class = serializers.SaveFileQuestionSerializer
         return_field_type = Question
@@ -770,10 +818,14 @@ class Answer(Node, graphene.Interface):
     question = graphene.Field(Question, required=True)
     meta = generic.GenericScalar(required=True)
 
+    resolve_question = suppressable_visibility_resolver()
+
     @classmethod
     def resolve_type(cls, instance, info):
         return resolve_answer(instance)
 
+    Meta = InterfaceMetaFactory()
+
 
 class AnswerQuerysetMixin(object):
     """Mixin to combine all different answer types into one queryset."""
@@ -788,7 +840,7 @@ class IntegerAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
 
     class Meta:
         model = models.Answer
-        exclude = ("document", "documents", "file", "date")
+        exclude = ("document", "documents", "files", "date")
         use_connection = False
         interfaces = (Answer, graphene.Node)
 
@@ -798,7 +850,7 @@ class FloatAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
 
     class Meta:
         model = models.Answer
-        exclude = ("document", "documents", "file", "date")
+        exclude = ("document", "documents", "files", "date")
         use_connection = False
         interfaces = (Answer, graphene.Node)
 
@@ -811,7 +863,7 @@ class DateAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
 
     class Meta:
         model = models.Answer
-        exclude = ("document", "documents", "file")
+        exclude = ("document", "documents", "files")
         use_connection = False
         interfaces = (Answer, graphene.Node)
 
@@ -832,11 +884,11 @@ class StringAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
 
     def resolve_selected_option(self, info, **args):
         selected_options = self.selected_options
-        return selected_options.first() if selected_options else None
+        return selected_options[0] if selected_options else None
 
     class Meta:
         model = models.Answer
-        exclude = ("document", "documents", "file", "date")
+        exclude = ("document", "documents", "files", "date")
         use_connection = False
         interfaces = (Answer, graphene.Node)
 
@@ -847,7 +899,7 @@ class ListAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
 
     class Meta:
         model = models.Answer
-        exclude = ("document", "documents", "file", "date")
+        exclude = ("document", "documents", "files", "date")
         use_connection = False
         interfaces = (Answer, graphene.Node)
 
@@ -858,10 +910,11 @@ class AnswerConnection(CountableConnectionBase):
 
 
 class Document(FormDjangoObjectType):
-    answers = DjangoFilterSetConnectionField(
+    answers = DjangoFilterInterfaceConnectionField(
         AnswerConnection,
         filterset_class=CollectionFilterSetFactory(
-            filters.AnswerFilterSet, orderset_class=filters.AnswerOrderSet
+            filterset_class=filters.AnswerFilterSet,
+            orderset_class=filters.AnswerOrderSet,
         ),
     )
     meta = generic.GenericScalar()
@@ -869,6 +922,11 @@ class Document(FormDjangoObjectType):
     modified_content_by_user = graphene.String()
     modified_content_by_group = graphene.String()
 
+    resolve_form = suppressable_visibility_resolver()
+    resolve_case = suppressable_visibility_resolver()
+    resolve_source = suppressable_visibility_resolver()
+    resolve_work_item = suppressable_visibility_resolver()
+
     class Meta:
         model = models.Document
         exclude = ("family", "dynamicoption_set")
@@ -884,7 +942,7 @@ class TableAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
 
     class Meta:
         model = models.Answer
-        exclude = ("documents", "file", "date")
+        exclude = ("documents", "files", "date")
         use_connection = False
         interfaces = (Answer, graphene.Node)
 
@@ -894,27 +952,30 @@ class File(FormDjangoObjectType):
     upload_url = graphene.String()
     download_url = graphene.String()
     metadata = generic.GenericScalar()
-    answer = graphene.Field("caluma.caluma_form.schema.FileAnswer")
 
     class Meta:
         model = models.File
         interfaces = (relay.Node,)
+        fields = "__all__"
 
 
-class FileAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
-    value = graphene.Field(File, required=True)
+class FilesAnswer(AnswerQuerysetMixin, FormDjangoObjectType):
+    value = graphene.List(File, required=True)
 
     def resolve_value(self, info, **args):
-        return self.file
+        return self.files.all()
 
     class Meta:
         model = models.Answer
-        exclude = ("document", "documents", "date")
+        exclude = ("document", "documents", "date", "files")
         use_connection = False
         interfaces = (Answer, graphene.Node)
 
 
 class SaveDocument(Mutation):
+    class Input:
+        id = graphene.String()
+
     class Meta:
         serializer_class = serializers.DocumentSerializer
         model_operations = ["create", "update"]
@@ -976,9 +1037,17 @@ class SaveDocumentTableAnswer(SaveDocumentAnswer):
         return_field_type = Answer
 
 
-class SaveDocumentFileAnswer(SaveDocumentAnswer):
+class SaveFile(graphene.InputObjectType):
+    id = graphene.String()
+    name = graphene.String()
+
+
+class SaveDocumentFilesAnswer(SaveDocumentAnswer):
+    class Input:
+        value = graphene.List(SaveFile, required=False)
+
     class Meta:
-        serializer_class = serializers.SaveDocumentFileAnswerSerializer
+        serializer_class = serializers.SaveDocumentFilesAnswerSerializer
         return_field_type = Answer
 
 
@@ -1071,7 +1140,7 @@ class Mutation(object):
     save_integer_question = SaveIntegerQuestion().Field()
     save_table_question = SaveTableQuestion().Field()
     save_form_question = SaveFormQuestion().Field()
-    save_file_question = SaveFileQuestion().Field()
+    save_files_question = SaveFilesQuestion().Field()
     save_static_question = SaveStaticQuestion().Field()
     save_calculated_float_question = SaveCalculatedFloatQuestion().Field()
     save_action_button_question = SaveActionButtonQuestion().Field()
@@ -1084,7 +1153,7 @@ class Mutation(object):
     save_document_date_answer = SaveDocumentDateAnswer().Field()
     save_document_list_answer = SaveDocumentListAnswer().Field()
     save_document_table_answer = SaveDocumentTableAnswer().Field()
-    save_document_file_answer = SaveDocumentFileAnswer().Field()
+    save_document_files_answer = SaveDocumentFilesAnswer().Field()
 
     save_default_string_answer = SaveDefaultStringAnswer().Field()
     save_default_integer_answer = SaveDefaultIntegerAnswer().Field()
@@ -1116,14 +1185,13 @@ class DocumentValidityConnection(CountableConnectionBase):
         node = ValidationResult
 
 
-def validate_document(info, document_global_id):
-
+def validate_document(info, document_global_id, **kwargs):
     document_id = extract_global_id(document_global_id)
 
     document_qs = Document.get_queryset(models.Document.objects.all(), info)
 
     document = get_object_or_404(document_qs, pk=document_id)
-    result = get_document_validity(document, info)
+    result = get_document_validity(document, info.context.user, **kwargs)
 
     errors = result.pop("errors")
     result = ValidationResult(
@@ -1137,36 +1205,44 @@ class Query:
     all_forms = DjangoFilterConnectionField(
         Form,
         filterset_class=CollectionFilterSetFactory(
-            filters.FormFilterSet, orderset_class=filters.FormOrderSet
+            filterset_class=filters.FormFilterSet,
+            orderset_class=filters.FormOrderSet,
         ),
     )
-    all_questions = DjangoFilterSetConnectionField(
+    all_questions = DjangoFilterInterfaceConnectionField(
         QuestionConnection,
         filterset_class=CollectionFilterSetFactory(
-            filters.QuestionFilterSet, orderset_class=filters.QuestionOrderSet
+            filterset_class=filters.QuestionFilterSet,
+            orderset_class=filters.QuestionOrderSet,
         ),
     )
     all_documents = DjangoFilterConnectionField(
         Document,
         filterset_class=CollectionFilterSetFactory(
-            filters.DocumentFilterSet, filters.DocumentOrderSet
+            filterset_class=filters.DocumentFilterSet,
+            orderset_class=filters.DocumentOrderSet,
         ),
     )
     all_format_validators = ConnectionField(FormatValidatorConnection)
     all_used_dynamic_options = DjangoFilterConnectionField(
         DynamicOption,
-        filterset_class=CollectionFilterSetFactory(filters.DynamicOptionFilterSet),
+        filterset_class=CollectionFilterSetFactory(
+            filterset_class=filters.DynamicOptionFilterSet,
+            orderset_class=filters.DynamicOptionOrderSet,
+        ),
     )
 
     document_validity = ConnectionField(
-        DocumentValidityConnection, id=graphene.ID(required=True)
+        DocumentValidityConnection,
+        id=graphene.ID(required=True),
+        data_source_context=graphene.JSONString(),
     )
 
-    def resolve_all_format_validators(self, info):
+    def resolve_all_format_validators(self, info, **kwargs):
         return get_format_validators()
 
-    def resolve_document_validity(self, info, id):
-        return validate_document(info, id)
+    def resolve_document_validity(self, info, id, **kwargs):
+        return validate_document(info, id, **kwargs)
 
 
 QUESTION_ANSWER_TYPES = {
@@ -1178,7 +1254,7 @@ QUESTION_ANSWER_TYPES = {
     models.Question.TYPE_TEXTAREA: StringAnswer,
     models.Question.TYPE_TEXT: StringAnswer,
     models.Question.TYPE_TABLE: TableAnswer,
-    models.Question.TYPE_FILE: FileAnswer,
+    models.Question.TYPE_FILES: FilesAnswer,
     models.Question.TYPE_DYNAMIC_CHOICE: StringAnswer,
     models.Question.TYPE_DYNAMIC_MULTIPLE_CHOICE: ListAnswer,
     models.Question.TYPE_CALCULATED_FLOAT: FloatAnswer,
@@ -1196,7 +1272,7 @@ QUESTION_OBJECT_TYPES = {
     models.Question.TYPE_DATE: DateQuestion,
     models.Question.TYPE_TABLE: TableQuestion,
     models.Question.TYPE_FORM: FormQuestion,
-    models.Question.TYPE_FILE: FileQuestion,
+    models.Question.TYPE_FILES: FilesQuestion,
     models.Question.TYPE_STATIC: StaticQuestion,
     models.Question.TYPE_CALCULATED_FLOAT: CalculatedFloatQuestion,
     models.Question.TYPE_ACTION_BUTTON: ActionButtonQuestion,
