@@ -3,10 +3,10 @@ from rest_framework import exceptions
 from rest_framework.serializers import (
     BooleanField,
     CharField,
-    ChoiceField,
     DateField,
     FloatField,
     IntegerField,
+    JSONField,
     ListField,
     PrimaryKeyRelatedField,
 )
@@ -18,15 +18,15 @@ from .jexl import QuestionJexl
 
 class QuestionJexlField(serializers.JexlField):
     def __init__(self, **kwargs):
-        super().__init__(QuestionJexl(), **kwargs)
+        super().__init__(QuestionJexl(field=None), **kwargs)
 
 
-class ButtonActionField(ChoiceField):
+class ButtonActionField(serializers.CalumaChoiceField):
     def __init__(self, **kwargs):
         super().__init__(models.Question.ACTION_CHOICES, **kwargs)
 
 
-class ButtonColorField(ChoiceField):
+class ButtonColorField(serializers.CalumaChoiceField):
     def __init__(self, **kwargs):
         super().__init__(models.Question.COLOR_CHOICES, **kwargs)
 
@@ -335,6 +335,7 @@ class SaveDynamicMultipleChoiceQuestionSerializer(SaveQuestionSerializer):
 class SaveFloatQuestionSerializer(SaveQuestionSerializer):
     min_value = FloatField(required=False, allow_null=True)
     max_value = FloatField(required=False, allow_null=True)
+    step = FloatField(required=False, allow_null=True)
 
     def validate(self, data):
         min_value = (
@@ -351,6 +352,10 @@ class SaveFloatQuestionSerializer(SaveQuestionSerializer):
                 f"max_value {max_value} is smaller than {min_value}"
             )
 
+        step = data.get("step")
+        if step and step < 0:
+            raise exceptions.ValidationError(f"step {step} is smaller than 0")
+
         data["type"] = models.Question.TYPE_FLOAT
         return super().validate(data)
 
@@ -358,6 +363,7 @@ class SaveFloatQuestionSerializer(SaveQuestionSerializer):
         fields = SaveQuestionSerializer.Meta.fields + [
             "min_value",
             "max_value",
+            "step",
             "placeholder",
             "hint_text",
         ]
@@ -424,7 +430,7 @@ class SaveFormQuestionSerializer(SaveQuestionSerializer):
 
 class SaveFileQuestionSerializer(SaveQuestionSerializer):
     def validate(self, data):
-        data["type"] = models.Question.TYPE_FILE
+        data["type"] = models.Question.TYPE_FILES
         return super().validate(data)
 
     class Meta(SaveQuestionSerializer.Meta):
@@ -468,6 +474,7 @@ class SaveActionButtonQuestionSerializer(SaveQuestionSerializer):
     action = ButtonActionField(required=True)
     color = ButtonColorField(required=True)
     validate_on_enter = BooleanField(required=True)
+    show_validation = BooleanField(required=True)
 
     def validate(self, data):
         data["type"] = models.Question.TYPE_ACTION_BUTTON
@@ -484,12 +491,13 @@ class SaveActionButtonQuestionSerializer(SaveQuestionSerializer):
             "action",
             "color",
             "validate_on_enter",
+            "show_validation",
         ]
 
 
 class SaveOptionSerializer(serializers.ModelSerializer):
     class Meta:
-        fields = ["slug", "label", "is_archived", "meta"]
+        fields = ["slug", "label", "is_archived", "is_hidden", "meta"]
         model = models.Option
 
 
@@ -525,9 +533,24 @@ class DocumentSerializer(serializers.ModelSerializer):
 
 
 class SaveAnswerSerializer(serializers.ModelSerializer):
+    document = serializers.GlobalIDPrimaryKeyRelatedField(
+        queryset=models.Document.objects, required=True
+    )
+    data_source_context = JSONField(
+        encoder=None,
+        required=False,
+        allow_null=True,
+        write_only=True,
+        help_text="JSON object passed as context to the data source of dynamic questions",
+    )
+
     def validate(self, data):
         data = domain_logic.SaveAnswerLogic.validate_for_save(
-            data, self.context["request"].user, self.instance, True
+            data,
+            self.context["request"].user,
+            self.instance,
+            True,
+            data.pop("data_source_context", None),
         )
         return super().validate(data)
 
@@ -543,7 +566,14 @@ class SaveAnswerSerializer(serializers.ModelSerializer):
 
     class Meta:
         model = models.Answer
-        fields = ["question", "document", "meta", "value"]
+        fields = [
+            "question",
+            "document",
+            "meta",
+            "value",
+            "question",
+            "data_source_context",
+        ]
 
 
 class SaveDocumentStringAnswerSerializer(SaveAnswerSerializer):
@@ -594,14 +624,15 @@ class SaveDocumentTableAnswerSerializer(SaveAnswerSerializer):
         pass
 
 
-class SaveDocumentFileAnswerSerializer(SaveAnswerSerializer):
-    value = CharField(write_only=True, source="file", required=False)
-    value_id = PrimaryKeyRelatedField(read_only=True, source="file", required=False)
+class SaveDocumentFilesAnswerSerializer(SaveAnswerSerializer):
+    value = ListField(
+        source="files",
+        required=False,
+        help_text="List of files for this answer",
+    )
 
     class Meta(SaveAnswerSerializer.Meta):
-        fields = SaveAnswerSerializer.Meta.fields + [
-            "value_id",
-        ]
+        fields = SaveAnswerSerializer.Meta.fields
 
 
 class RemoveAnswerSerializer(serializers.ModelSerializer):
