@@ -1,4 +1,3 @@
-import base64
 import functools
 import hashlib
 
@@ -7,8 +6,9 @@ from django.conf import settings
 from django.core.cache import cache
 from django.core.exceptions import ImproperlyConfigured
 from django.http.response import HttpResponse
-from django.utils.encoding import force_bytes, smart_text
+from django.utils.encoding import force_bytes, smart_str
 from django.utils.module_loading import import_string
+from graphene.validation import DisableIntrospection, depth_limit_validator
 from graphene_django.views import GraphQLView, HttpError
 from rest_framework.authentication import get_authorization_header
 
@@ -19,7 +19,48 @@ class HttpResponseUnauthorized(HttpResponse):
     status_code = 401
 
 
+class SuppressIntrospection(DisableIntrospection):
+    """Validate request to reject any introspection, except the bare minimum."""
+
+    # The base class, graphene.validation.DisableIntrospection, is too strict:
+    # It rejects everything starting with double underscores (as per GQL spec)
+    # but we need `__typename` for our frontends to work correctly.
+
+    ALLOWED_INTROSPECTION_KEYS = ["__typename"]
+
+    def enter_field(self, node, *_args):
+        field_name = node.name.value
+        if field_name not in self.ALLOWED_INTROSPECTION_KEYS:
+            super().enter_field(node, *_args)
+
+
+custom_validation_rules = []
+if settings.DISABLE_INTROSPECTION:  # pragma: no cover
+    custom_validation_rules.append(SuppressIntrospection)
+
+if settings.QUERY_DEPTH_LIMIT:  # pragma: no cover
+    custom_validation_rules.append(
+        depth_limit_validator(max_depth=settings.QUERY_DEPTH_LIMIT)
+    )
+
+
 class AuthenticationGraphQLView(GraphQLView):
+    @classmethod
+    def _requests_session(cls):
+        """
+        Return a requests session that's kept alive over time.
+
+        We use a "global" requests session for auth, as the initialisation of
+        SSL context and re-opening of a TCP connection to the OIDC IDP can cause
+        some additional load (and work time) that we want to avoid if possible.
+        """
+        if not hasattr(cls, "_http_client"):
+            setattr(cls, "_http_client", requests.Session())
+        return cls._http_client
+
+    if custom_validation_rules:  # pragma: no cover
+        validation_rules = tuple(custom_validation_rules)
+
     def get_bearer_token(self, request):
         auth = get_authorization_header(request).split()
         header_prefix = "Bearer"
@@ -27,7 +68,7 @@ class AuthenticationGraphQLView(GraphQLView):
         if not auth:
             return None
 
-        if smart_text(auth[0].lower()) != header_prefix.lower():
+        if smart_str(auth[0].lower()) != header_prefix.lower():
             raise HttpError(HttpResponseUnauthorized("No Bearer Authorization header"))
 
         if len(auth) == 1:
@@ -43,29 +84,10 @@ class AuthenticationGraphQLView(GraphQLView):
         return auth[1]
 
     def get_userinfo(self, token):
-        response = requests.get(
+        response = self._requests_session().get(
             settings.OIDC_USERINFO_ENDPOINT,
             verify=settings.OIDC_VERIFY_SSL,
-            headers={"Authorization": f"Bearer {smart_text(token)}"},
-        )
-        response.raise_for_status()
-        return response.json()
-
-    def get_introspection(self, token):
-        basic = base64.b64encode(
-            f"{settings.OIDC_INTROSPECT_CLIENT_ID}:{settings.OIDC_INTROSPECT_CLIENT_SECRET}".encode(
-                "utf-8"
-            )
-        ).decode()
-        headers = {
-            "Authorization": f"Basic {basic}",
-            "Content-Type": "application/x-www-form-urlencoded",
-        }
-        response = requests.post(
-            settings.OIDC_INTROSPECT_ENDPOINT,
-            verify=settings.OIDC_VERIFY_SSL,
-            headers=headers,
-            data={"token": token},
+            headers={"Authorization": f"Bearer {smart_str(token)}"},
         )
         response.raise_for_status()
         return response.json()
@@ -86,41 +108,22 @@ class AuthenticationGraphQLView(GraphQLView):
             )
 
         userinfo_method = functools.partial(self.get_userinfo, token=token)
-        # token might be too long for key so we use hash sum instead.
+        # token might be too long for key, so we use hash sum instead.
         hashsum_token = hashlib.sha256(force_bytes(token)).hexdigest()
 
         try:
-            userinfo = cache.get_or_set(
+            claims = cache.get_or_set(
                 f"authentication.userinfo.{hashsum_token}",
                 userinfo_method,
                 timeout=settings.OIDC_BEARER_TOKEN_REVALIDATION_TIME,
             )
-            return self._oidc_user(token=token, userinfo=userinfo)
         except requests.HTTPError as e:
-            try:
-                if (
-                    e.response.status_code in [401, 403]
-                    and settings.OIDC_INTROSPECT_ENDPOINT
-                ):
-                    introspect_method = functools.partial(
-                        self.get_introspection, token=token
-                    )
-                    introspection = cache.get_or_set(
-                        f"authentication.introspect.{hashsum_token}",
-                        introspect_method,
-                        timeout=settings.OIDC_BEARER_TOKEN_REVALIDATION_TIME,
-                    )
-                    if "client_id" not in introspection:
-                        response = HttpResponse(status=401)
-                        raise HttpError(response)
-                    return self._oidc_user(token=token, introspection=introspection)
-                else:
-                    raise e
-            except requests.HTTPError as internal_exception:
-                # convert request error to django http error
-                response = HttpResponse()
-                response.status_code = internal_exception.response.status_code
-                raise HttpError(response, message=str(internal_exception))
+            # convert request error to django http error
+            response = HttpResponse()
+            response.status_code = e.response.status_code
+            raise HttpError(response, message=str(e))
+
+        return self._oidc_user(token=token, claims=claims)
 
     def dispatch(self, request, *args, **kwargs):
         try:
