@@ -1,4 +1,4 @@
-from typing import Optional
+from typing import List, Optional
 
 from caluma.caluma_form.models import Form
 from caluma.caluma_user.models import BaseUser
@@ -13,7 +13,7 @@ def start_case(
     form: Optional[Form] = None,
     parent_work_item: Optional[models.WorkItem] = None,
     context: Optional[dict] = None,
-    **kwargs
+    **kwargs,
 ) -> models.Case:
     """
     Start a case of a given workflow (just like `saveCase`).
@@ -28,12 +28,12 @@ def start_case(
     ... )
     <Case: Case object (some-uuid)>
     """
-    data = {"workflow": workflow, "form": form, "parent_work_item": parent_work_item}
+    data = {"workflow": workflow, "form": form}
 
     data.update(kwargs)
 
     validated_data = domain_logic.StartCaseLogic.pre_start(
-        domain_logic.StartCaseLogic.validate_for_start(data), user
+        domain_logic.StartCaseLogic.validate_for_start(data), user, parent_work_item
     )
 
     case = models.Case.objects.create(**validated_data)
@@ -54,7 +54,9 @@ def complete_work_item(
     <WorkItem: WorkItem object (some-uuid)>
     ```
     """
-    domain_logic.CompleteWorkItemLogic.validate_for_complete(work_item, user)
+    domain_logic.CompleteWorkItemLogic.validate_for_complete(
+        work_item, user, context=context
+    )
     validated_data = domain_logic.CompleteWorkItemLogic.pre_complete(
         work_item, {}, user, context
     )
@@ -210,6 +212,34 @@ def resume_case(
     return case
 
 
+def reopen_case(
+    case: models.Case,
+    work_items: List[models.WorkItem],
+    user: BaseUser,
+    context: Optional[dict] = None,
+) -> models.Case:
+    """
+    Reopen a case (just like `ReopenCase`).
+
+    >>> reopen_case(
+    ...     case=models.Case.first(),
+    ...     work_items=[...]
+    ...     user=AnonymousUser()
+    ... )
+    <Case: Case object (some-uuid)>
+    """
+
+    domain_logic.ReopenCaseLogic.validate_for_reopen(case, work_items)
+
+    domain_logic.ReopenCaseLogic.pre_reopen(case, work_items, user, context)
+
+    domain_logic.ReopenCaseLogic.do_reopen(case, work_items)
+
+    domain_logic.ReopenCaseLogic.post_reopen(case, work_items, user, context)
+
+    return case
+
+
 def resume_work_item(
     work_item: models.WorkItem, user: BaseUser, context: Optional[dict] = None
 ) -> models.WorkItem:
@@ -233,3 +263,30 @@ def resume_work_item(
     domain_logic.ResumeWorkItemLogic.post_resume(work_item, user, context)
 
     return work_item
+
+
+def redo_work_item(
+    work_item: models.WorkItem, user: BaseUser, context: Optional[dict] = None
+) -> models.WorkItem:
+    """
+    Redo a work item.
+
+    >>> redo_work_item(
+    ...     work_item=models.WorkItem.objects.first(),
+    ...     user=AnonymousUser()
+    ... )
+    <WorkItem: WorkItem object (some-uuid)>
+    """
+    domain_logic.RedoWorkItemLogic.validate_for_redo(work_item)
+
+    validated_data = domain_logic.RedoWorkItemLogic.pre_redo(
+        work_item, {}, user, context
+    )
+
+    domain_logic.RedoWorkItemLogic.set_succeeding_work_item_status_redo(work_item)
+
+    update_model(work_item, validated_data)
+
+    domain_logic.RedoWorkItemLogic.post_redo(work_item, user, context)
+
+    return work_item
