@@ -1,10 +1,10 @@
 from django.core.exceptions import ValidationError
 from django.utils import timezone
 
+from caluma.caluma_core.events import send_event
 from caluma.caluma_form.domain_logic import SaveDocumentLogic
 
-from . import api, models, utils, validators
-from .events import send_event_with_deprecations
+from . import api, events, models, utils, validators
 
 
 class StartCaseLogic:
@@ -35,9 +35,9 @@ class StartCaseLogic:
         return data
 
     @staticmethod
-    def pre_start(validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_create_case",
+    def pre_start(validated_data, user, parent_work_item=None, context=None):
+        send_event(
+            events.pre_create_case,
             sender="pre_create_case",
             case=None,
             validated_data=validated_data,
@@ -45,7 +45,6 @@ class StartCaseLogic:
             context=context,
         )
 
-        parent_work_item = validated_data.get("parent_work_item")
         validated_data["status"] = models.Case.STATUS_RUNNING
 
         form = validated_data.pop("form", None)
@@ -75,8 +74,8 @@ class StartCaseLogic:
 
         work_items = utils.create_work_items(tasks, case, user, None, context)
 
-        send_event_with_deprecations(
-            "post_create_case",
+        send_event(
+            events.post_create_case,
             sender="case_post_create",
             case=case,
             user=user,
@@ -84,8 +83,8 @@ class StartCaseLogic:
         )
 
         for work_item in work_items:  # pragma: no cover
-            send_event_with_deprecations(
-                "post_create_work_item",
+            send_event(
+                events.post_create_work_item,
                 sender="case_post_create",
                 work_item=work_item,
                 user=user,
@@ -98,8 +97,9 @@ class StartCaseLogic:
 class CompleteWorkItemLogic:
     @staticmethod
     def _can_continue(work_item, task):
-        # If a "multiple instance" task has running siblings, the task is not completed
-        if task.is_multiple_instance:
+        # If a multiple instance work item has running siblings the workflow can
+        # only continue if the task is allowed to continue asynchronously
+        if task.is_multiple_instance and not task.continue_async:
             return not work_item.case.work_items.filter(
                 task=task, status=models.WorkItem.STATUS_READY
             ).exists()
@@ -112,7 +112,7 @@ class CompleteWorkItemLogic:
         ).exists()
 
     @staticmethod
-    def validate_for_complete(work_item, user):
+    def validate_for_complete(work_item, user, context=None):
         validators.WorkItemValidator().validate(
             status=work_item.status,
             child_case=work_item.child_case,
@@ -120,6 +120,8 @@ class CompleteWorkItemLogic:
             case=work_item.case,
             document=work_item.document,
             user=user,
+            validation_context=context.get("validation_context") if context else None,
+            data_source_context=context.get("data_source_context") if context else None,
         )
 
     @staticmethod
@@ -128,10 +130,10 @@ class CompleteWorkItemLogic:
         validated_data,
         user,
         context=None,
-        event="pre_complete_work_item",
+        event=events.pre_complete_work_item,
         sender="pre_complete_work_item",
     ):
-        send_event_with_deprecations(
+        send_event(
             event, sender=sender, work_item=work_item, user=user, context=context
         )
 
@@ -147,12 +149,12 @@ class CompleteWorkItemLogic:
         work_item,
         user,
         context=None,
-        event="post_complete_work_item",
+        event=events.post_complete_work_item,
         sender="post_complete_work_item",
     ):
         case = work_item.case
 
-        send_event_with_deprecations(
+        send_event(
             event, sender=sender, work_item=work_item, user=user, context=context
         )
 
@@ -180,12 +182,16 @@ class CompleteWorkItemLogic:
 
             if all_siblings_complete:
                 created_work_items = utils.create_work_items(
-                    next_tasks, case, user, work_item, context
+                    next_tasks,
+                    case,
+                    user,
+                    work_item,
+                    context,
                 )
 
                 for created_work_item in created_work_items:  # pragma: no cover
-                    send_event_with_deprecations(
-                        "post_create_work_item",
+                    send_event(
+                        events.post_create_work_item,
                         sender=sender,
                         work_item=created_work_item,
                         user=user,
@@ -203,10 +209,17 @@ class CompleteWorkItemLogic:
             case.closed_at = timezone.now()
             case.closed_by_user = user.username
             case.closed_by_group = user.group
-            case.save()
+            case.save(
+                update_fields=[
+                    "status",
+                    "closed_at",
+                    "closed_by_user",
+                    "closed_by_group",
+                ]
+            )
 
-            send_event_with_deprecations(
-                "post_complete_case",
+            send_event(
+                events.post_complete_case,
                 sender=sender,
                 case=case,
                 user=user,
@@ -232,7 +245,7 @@ class SkipWorkItemLogic:
             validated_data,
             user,
             context,
-            event="pre_skip_work_item",
+            event=events.pre_skip_work_item,
             sender="pre_skip_work_item",
         )
 
@@ -252,7 +265,7 @@ class SkipWorkItemLogic:
             work_item,
             user,
             context,
-            event="post_skip_work_item",
+            event=events.post_skip_work_item,
             sender="post_skip_work_item",
         )
 
@@ -280,8 +293,8 @@ class CancelCaseLogic:
 
     @staticmethod
     def pre_cancel(case, validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_cancel_case",
+        send_event(
+            events.pre_cancel_case,
             sender="pre_cancel_case",
             case=case,
             user=user,
@@ -302,8 +315,8 @@ class CancelCaseLogic:
 
     @staticmethod
     def post_cancel(case, user, context=None):
-        send_event_with_deprecations(
-            "post_cancel_case",
+        send_event(
+            events.post_cancel_case,
             sender="post_cancel_case",
             case=case,
             user=user,
@@ -331,8 +344,8 @@ class SuspendCaseLogic:
 
     @staticmethod
     def pre_suspend(case, validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_suspend_case",
+        send_event(
+            events.pre_suspend_case,
             sender="pre_suspend_case",
             case=case,
             user=user,
@@ -348,8 +361,8 @@ class SuspendCaseLogic:
 
     @staticmethod
     def post_suspend(case, user, context=None):
-        send_event_with_deprecations(
-            "post_suspend_case",
+        send_event(
+            events.post_suspend_case,
             sender="post_suspend_case",
             case=case,
             user=user,
@@ -359,6 +372,78 @@ class SuspendCaseLogic:
         return case
 
 
+class ReopenCaseLogic:
+    """
+    Shared domain logic for reopening cases.
+
+    Used in the `reopenCase` mutation and in the `reopen_case` API. The logic
+    for case reopening is split in three parts (`validate_for_reopen`,
+    `pre_reopen` and `post_reopen`) so that in between the appropriate update
+    method can be called (`super().update(...)` for the serializer and
+    `Case.objects.update(...) for the python API`).
+    """
+
+    @staticmethod
+    def validate_for_reopen(case, work_items):
+        if case.status not in [
+            models.Case.STATUS_COMPLETED,
+            models.Case.STATUS_CANCELED,
+        ]:
+            raise ValidationError("Only completed and canceled cases can be reopened.")
+
+        if (
+            case.family != case
+            and case.parent_work_item.status != models.WorkItem.STATUS_READY
+        ):
+            raise ValidationError(
+                "Only child cases of ready work items can be reopened."
+            )
+
+        for work_item in work_items:
+            if work_item.succeeding_work_items.exclude(
+                status=models.WorkItem.STATUS_REDO
+            ).exists():
+                raise ValidationError(
+                    "Only work items at the end of a branch can be reopened."
+                )
+
+            if work_item.case != case:
+                raise ValidationError(
+                    "Only work items belonging to the case specified can be reopend."
+                )
+
+    @staticmethod
+    def pre_reopen(case, work_items, user, context=None):
+        send_event(
+            events.pre_reopen_case,
+            sender="pre_reopen_case",
+            case=case,
+            user=user,
+            work_items=work_items,
+            context=context,
+        )
+
+    @staticmethod
+    def post_reopen(case, work_items, user, context=None):
+        send_event(
+            events.post_reopen_case,
+            sender="post_reopen_case",
+            case=case,
+            user=user,
+            work_items=work_items,
+            context=context,
+        )
+
+    @staticmethod
+    def do_reopen(case, work_items):
+        case.status = models.Case.STATUS_RUNNING
+        case.save()
+
+        for work_item in work_items:
+            work_item.status = models.WorkItem.STATUS_READY
+            work_item.save()
+
+
 class ResumeCaseLogic:
     """
     Shared domain logic for resuming cases.
@@ -377,8 +462,8 @@ class ResumeCaseLogic:
 
     @staticmethod
     def pre_resume(case, validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_resume_case",
+        send_event(
+            events.pre_resume_case,
             sender="pre_resume_case",
             case=case,
             user=user,
@@ -396,8 +481,8 @@ class ResumeCaseLogic:
 
     @staticmethod
     def post_resume(case, user, context=None):
-        send_event_with_deprecations(
-            "post_resume_case",
+        send_event(
+            events.post_resume_case,
             sender="post_resume_case",
             case=case,
             user=user,
@@ -418,8 +503,8 @@ class CancelWorkItemLogic:
 
     @staticmethod
     def pre_cancel(work_item, validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_cancel_work_item",
+        send_event(
+            events.pre_cancel_work_item,
             sender="pre_cancel_work_item",
             work_item=work_item,
             user=user,
@@ -441,8 +526,8 @@ class CancelWorkItemLogic:
 
     @staticmethod
     def post_cancel(work_item, user, context=None):
-        send_event_with_deprecations(
-            "post_cancel_work_item",
+        send_event(
+            events.post_cancel_work_item,
             sender="post_cancel_work_item",
             work_item=work_item,
             user=user,
@@ -460,8 +545,8 @@ class SuspendWorkItemLogic:
 
     @staticmethod
     def pre_suspend(work_item, validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_suspend_work_item",
+        send_event(
+            events.pre_suspend_work_item,
             sender="pre_suspend_work_item",
             work_item=work_item,
             user=user,
@@ -480,8 +565,8 @@ class SuspendWorkItemLogic:
 
     @staticmethod
     def post_suspend(work_item, user, context=None):
-        send_event_with_deprecations(
-            "post_suspend_work_item",
+        send_event(
+            events.post_suspend_work_item,
             sender="post_suspend_work_item",
             work_item=work_item,
             user=user,
@@ -499,8 +584,8 @@ class ResumeWorkItemLogic:
 
     @staticmethod
     def pre_resume(work_item, validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_resume_work_item",
+        send_event(
+            events.pre_resume_work_item,
             sender="pre_resume_work_item",
             work_item=work_item,
             user=user,
@@ -519,8 +604,8 @@ class ResumeWorkItemLogic:
 
     @staticmethod
     def post_resume(work_item, user, context=None):
-        send_event_with_deprecations(
-            "post_resume_work_item",
+        send_event(
+            events.post_resume_work_item,
             sender="post_resume_work_item",
             work_item=work_item,
             user=user,
@@ -544,13 +629,18 @@ class RedoWorkItemLogic:
         if work_item.status == models.WorkItem.STATUS_READY:
             raise ValidationError("Ready work items can't be redone.")
 
+        if not cls.is_work_item_redoable(work_item):
+            raise ValidationError("Workflow doesn't allow to redo this work item.")
+
+    @classmethod
+    def is_work_item_redoable(cls, work_item):
         for wi in cls._find_ready_in_work_item_tree(
             work_item, allowed_states=[models.WorkItem.STATUS_READY]
         ):
             if work_item.task in wi.get_redoable():
-                return
+                return True
 
-        raise ValidationError("Workflow doesn't allow to redo this work item.")
+        return False
 
     @classmethod
     def set_succeeding_work_item_status_redo(cls, work_item):
@@ -571,8 +661,8 @@ class RedoWorkItemLogic:
 
     @staticmethod
     def pre_redo(work_item, validated_data, user, context=None):
-        send_event_with_deprecations(
-            "pre_redo_work_item",
+        send_event(
+            events.pre_redo_work_item,
             sender="pre_redo_work_item",
             work_item=work_item,
             user=user,
@@ -585,8 +675,8 @@ class RedoWorkItemLogic:
 
     @staticmethod
     def post_redo(work_item, user, context=None):
-        send_event_with_deprecations(
-            "post_redo_work_item",
+        send_event(
+            events.post_redo_work_item,
             sender="post_redo_work_item",
             work_item=work_item,
             user=user,
