@@ -1,22 +1,23 @@
 import graphene
-from django.db.models import OuterRef, Subquery
-from django_filters.rest_framework import BooleanFilter
+from django_filters.rest_framework import (
+    BooleanFilter,
+    DateTimeFilter,
+    MultipleChoiceFilter,
+)
 
 from ..caluma_core.filters import (
+    BaseFilterSet,
     CharFilter,
-    FilterSet,
     GlobalIDFilter,
+    GlobalIDMultipleChoiceFilter,
     JSONValueFilter,
     MetaFilterSet,
-    OrderingFilter,
     SearchFilter,
-    SlugMultipleChoiceFilter,
     StringListFilter,
     generate_list_filter_class,
 )
 from ..caluma_core.ordering import AttributeOrderingFactory, MetaFieldOrdering
 from ..caluma_form.filters import HasAnswerFilter, SearchAnswersFilter
-from ..caluma_form.models import Answer, Question
 from ..caluma_form.ordering import AnswerValueOrdering
 from . import models
 
@@ -41,21 +42,20 @@ def case_status_filter(*args, **kwargs):
 
 class WorkflowFilterSet(MetaFilterSet):
     search = SearchFilter(fields=("slug", "name", "description"))
-    order_by = OrderingFilter(label="WorkflowOrdering", fields=("name", "description"))
 
     class Meta:
         model = models.Workflow
         fields = ("slug", "name", "description", "is_published", "is_archived")
 
 
-class WorkflowOrderSet(FilterSet):
+class WorkflowOrderSet(BaseFilterSet):
     meta = MetaFieldOrdering()
     attribute = AttributeOrderingFactory(
         models.Workflow,
         fields=[
+            "created_at",
+            "modified_at",
             "allow_all_forms",
-            "created_by_group",
-            "created_by_user",
             "description",
             "is_archived",
             "is_published",
@@ -69,7 +69,7 @@ class WorkflowOrderSet(FilterSet):
         fields = ("meta", "attribute")
 
 
-class FlowFilterSet(FilterSet):
+class FlowFilterSet(BaseFilterSet):
     task = GlobalIDFilter(field_name="task_flows__task")
 
     class Meta:
@@ -77,12 +77,23 @@ class FlowFilterSet(FilterSet):
         fields = ("task",)
 
 
+class FlowOrderSet(BaseFilterSet):
+    meta = MetaFieldOrdering()
+    attribute = AttributeOrderingFactory(
+        models.Flow,
+        fields=["created_at", "modified_at", "task"],
+    )
+
+    class Meta:
+        model = models.Flow
+        fields = ("meta", "attribute")
+
+
 class CaseFilterSet(MetaFilterSet):
-    id = GlobalIDFilter()
-    order_by = OrderingFilter(label="CaseOrdering", fields=("status",))
+    ids = GlobalIDMultipleChoiceFilter(field_name="pk")
 
     document_form = CharFilter(field_name="document__form_id")
-    document_forms = SlugMultipleChoiceFilter(field_name="document__form_id")
+    document_forms = MultipleChoiceFilter(field_name="document__form_id")
     has_answer = HasAnswerFilter(document_id="document__pk")
     work_item_document_has_answer = HasAnswerFilter(
         document_id="work_items__document__pk"
@@ -90,65 +101,28 @@ class CaseFilterSet(MetaFilterSet):
     root_case = GlobalIDFilter(field_name="family")
     search_answers = SearchAnswersFilter(document_id="document__pk")
     status = case_status_filter(lookup_expr="in")
-    order_by_question_answer_value = CharFilter(
-        method="filter_order_by_question_answer_value",
-        label=(
-            "Expects a question slug. If the slug is prefixed with a hyphen, "
-            "the order will be reversed\n\n"
-            "For file questions, the filename is used for sorting.\n\n"
-            "Table questions are not supported at this time."
-        ),
-    )
+    exclude_child_cases = BooleanFilter(method="filter_exclude_child_cases")
 
-    @staticmethod
-    def filter_order_by_question_answer_value(queryset, _, question_slug):
-        order_by = "-order_value" if question_slug.startswith("-") else "order_value"
-        question_slug = question_slug.lstrip("-")
+    def filter_exclude_child_cases(self, queryset, name, value):
+        if not value:
+            return queryset
 
-        # Based on question type, set answer field to use for sorting
-        not_supported = (Question.TYPE_TABLE,)
-        question = Question.objects.get(slug=question_slug)
-        answer_value = "value"
-        if question.type in not_supported:
-            raise RuntimeError(
-                f'Questions with type "{question.type}" are not supported '
-                f'by "filterOrderByQuestionAnswerValue"'
-            )
-        elif question.type == Question.TYPE_DATE:
-            answer_value = "date"
-        elif question.type == Question.TYPE_FILE:
-            answer_value = "file__name"
-
-        # Initialize subquery
-        answers = Answer.objects.filter(
-            question=question, document=OuterRef("document")
-        )
-
-        # Annotate the cases in the queryset with the value of the answer of the given
-        # question and order by it.
-        return queryset.annotate(
-            order_value=Subquery(answers.values(answer_value)[:1])
-        ).order_by(order_by)
+        return queryset.filter(parent_work_item__isnull=True)
 
     class Meta:
         model = models.Case
         fields = ("workflow",)
 
 
-class CaseOrderSet(FilterSet):
+class CaseOrderSet(BaseFilterSet):
     meta = MetaFieldOrdering()
     attribute = AttributeOrderingFactory(
         models.Case,
         fields=[
-            "allow_all_forms",
-            "created_by_group",
-            "created_by_user",
-            "description",
-            "is_archived",
-            "is_published",
-            "name",
+            "created_at",
+            "modified_at",
             "status",
-            "slug",
+            "document__form__name",
         ],
     )
     document_answer = AnswerValueOrdering(document_via="document")
@@ -160,28 +134,24 @@ class CaseOrderSet(FilterSet):
 
 class TaskFilterSet(MetaFilterSet):
     search = SearchFilter(fields=("slug", "name", "description"))
-    order_by = OrderingFilter(
-        label="TaskOrdering", fields=("name", "description", "type")
-    )
+    slugs = MultipleChoiceFilter(field_name="slug")
 
     class Meta:
         model = models.Task
-        fields = ("slug", "name", "description", "type", "is_archived")
+        fields = ("slug", "slugs", "name", "description", "type", "is_archived")
 
 
-class TaskOrderSet(FilterSet):
+class TaskOrderSet(BaseFilterSet):
     meta = MetaFieldOrdering()
     attribute = AttributeOrderingFactory(
         models.Task,
         fields=[
-            "allow_all_forms",
+            "created_at",
+            "modified_at",
             "lead_time",
             "type",
-            "created_by_group",
-            "created_by_user",
             "description",
             "is_archived",
-            "is_published",
             "name",
             "slug",
         ],
@@ -194,17 +164,29 @@ class TaskOrderSet(FilterSet):
 
 class WorkItemFilterSet(MetaFilterSet):
     id = GlobalIDFilter()
-    order_by = OrderingFilter(label="WorkItemOrdering", fields=("status", "deadline"))
     addressed_groups = StringListFilter(lookup_expr="overlap")
     controlling_groups = StringListFilter(lookup_expr="overlap")
     assigned_users = StringListFilter(lookup_expr="overlap")
 
     document_has_answer = HasAnswerFilter(document_id="document__pk")
     case_document_has_answer = HasAnswerFilter(document_id="case__document__pk")
+    case_document_forms = MultipleChoiceFilter(field_name="case__document__form_id")
     case_meta_value = JSONValueFilter(field_name="case__meta")
     root_case_meta_value = JSONValueFilter(field_name="case__family__meta")
+    case_search_answers = SearchAnswersFilter(document_id="case__document__pk")
 
-    tasks = SlugMultipleChoiceFilter(field_name="task_id")
+    deadline_before = DateTimeFilter(
+        field_name="deadline",
+        lookup_expr="lt",
+        label="Only return work items whose deadline is set to before the given datetime",
+    )
+    deadline_after = DateTimeFilter(
+        field_name="deadline",
+        lookup_expr="gte",
+        label="Only return work items whose deadline is set to after the given datetime",
+    )
+
+    tasks = MultipleChoiceFilter(field_name="task_id")
 
     has_deadline = BooleanFilter(
         field_name="deadline", lookup_expr="isnull", exclude=True
@@ -228,25 +210,21 @@ class WorkItemFilterSet(MetaFilterSet):
         )
 
 
-class WorkItemOrderSet(FilterSet):
+class WorkItemOrderSet(BaseFilterSet):
     meta = MetaFieldOrdering()
     case_meta = MetaFieldOrdering(field_name="case__meta")
     attribute = AttributeOrderingFactory(
         models.WorkItem,
         fields=[
-            "allow_all_forms",
-            "created_by_group",
-            "created_by_user",
-            "description",
             "created_at",
             "modified_at",
             "closed_at",
-            "is_archived",
-            "is_published",
+            "description",
             "name",
             "deadline",
             "status",
             "slug",
+            "case__document__form__name",
         ],
     )
     document_answer = AnswerValueOrdering(document_via="document")
