@@ -8,7 +8,7 @@ from django.dispatch import receiver
 from django.utils import timezone
 from localized_fields.fields import LocalizedField
 
-from ..caluma_core.models import ChoicesCharField, SlugModel, UUIDModel
+from ..caluma_core.models import SlugModel, UUIDModel
 
 
 class Task(SlugModel):
@@ -25,7 +25,7 @@ class Task(SlugModel):
 
     name = LocalizedField(blank=False, null=False, required=False)
     description = LocalizedField(blank=True, null=True, required=False)
-    type = ChoicesCharField(choices=TYPE_CHOICES_TUPLE, max_length=50)
+    type = models.CharField(choices=TYPE_CHOICES_TUPLE, max_length=50)
     meta = models.JSONField(default=dict)
     address_groups = models.TextField(
         blank=True,
@@ -54,6 +54,10 @@ class Task(SlugModel):
         default=False,
         help_text="Allows creating multiple work items for this task using the `CreateWorkItem` mutation. If true, one work item will be created for each entry in `address_groups`.",
     )
+    continue_async = models.BooleanField(
+        default=False,
+        help_text="Whether to continue the flow if the multiple instance work item has ready siblings",
+    )
 
     def calculate_deadline(self):
         if self.lead_time is not None:
@@ -145,7 +149,7 @@ class Case(UUIDModel):
     workflow = models.ForeignKey(
         Workflow, related_name="cases", on_delete=models.DO_NOTHING
     )
-    status = ChoicesCharField(choices=STATUS_CHOICE_TUPLE, max_length=50, db_index=True)
+    status = models.CharField(choices=STATUS_CHOICE_TUPLE, max_length=50, db_index=True)
     meta = models.JSONField(default=dict)
     document = models.OneToOneField(
         "caluma_form.Document",
@@ -212,7 +216,7 @@ class WorkItem(UUIDModel):
     task = models.ForeignKey(
         Task, on_delete=models.DO_NOTHING, related_name="work_items"
     )
-    status = ChoicesCharField(choices=STATUS_CHOICE_TUPLE, max_length=50, db_index=True)
+    status = models.CharField(choices=STATUS_CHOICE_TUPLE, max_length=50, db_index=True)
     meta = models.JSONField(default=dict)
 
     addressed_groups = ArrayField(
@@ -291,9 +295,13 @@ class WorkItem(UUIDModel):
 
     class Meta:
         indexes = [
+            models.Index(fields=["status"]),
             GinIndex(fields=["addressed_groups"]),
+            GinIndex(fields=["controlling_groups"]),
             GinIndex(fields=["assigned_users"]),
             GinIndex(fields=["meta"]),
+            models.Index(fields=["created_at"]),
+            models.Index(fields=["deadline"]),
         ]
 
 
