@@ -6,13 +6,22 @@ from graphene import relay
 from graphene.types import generic
 from graphene_django.rest_framework import serializer_converter
 
+from caluma.utils import suppressable_visibility_resolver
+
 from ..caluma_core.filters import (
     CollectionFilterSetFactory,
     DjangoFilterConnectionField,
-    DjangoFilterSetConnectionField,
+    DjangoFilterInterfaceConnectionField,
+    InterfaceMetaFactory,
 )
 from ..caluma_core.mutation import Mutation, UserDefinedPrimaryKeyMixin
-from ..caluma_core.types import CountableConnectionBase, DjangoObjectType, Node
+from ..caluma_core.types import (
+    CountableConnectionBase,
+    DjangoObjectType,
+    Node,
+    enum_type_from_field,
+)
+from ..caluma_workflow.domain_logic import RedoWorkItemLogic
 from . import filters, jexl, models, serializers
 
 
@@ -54,6 +63,17 @@ serializer_converter.get_graphene_type_from_serializer_field.register(
 )
 
 
+CaseStatus = enum_type_from_field(
+    "CaseStatus", models.Case.status, serializer_field=serializers.CaseStatusField
+)
+
+WorkItemStatus = enum_type_from_field(
+    "WorkItemStatus",
+    models.WorkItem.status,
+    serializer_field=serializers.WorkItemStatusField,
+)
+
+
 class Task(Node, graphene.Interface):
     id = graphene.ID(required=True)
     created_at = graphene.DateTime(required=True)
@@ -68,6 +88,7 @@ class Task(Node, graphene.Interface):
     control_groups = GroupJexl()
     meta = generic.GenericScalar(required=True)
     is_multiple_instance = graphene.Boolean(required=True)
+    continue_async = graphene.Boolean()
 
     @classmethod
     def resolve_type(cls, instance, info):
@@ -79,6 +100,9 @@ class Task(Node, graphene.Interface):
 
         return TASK_TYPE[instance.type]
 
+    resolve_form = suppressable_visibility_resolver()
+    Meta = InterfaceMetaFactory()
+
 
 class TaskConnection(CountableConnectionBase):
     class Meta:
@@ -96,7 +120,7 @@ class TaskQuerysetMixin(object):
 class SimpleTask(TaskQuerysetMixin, DjangoObjectType):
     class Meta:
         model = models.Task
-        exclude = ("task_flows", "work_items", "form")
+        exclude = ("task_flows", "work_items", "form", "type")
         use_connection = False
         interfaces = (Task, relay.Node)
 
@@ -104,7 +128,7 @@ class SimpleTask(TaskQuerysetMixin, DjangoObjectType):
 class CompleteWorkflowFormTask(TaskQuerysetMixin, DjangoObjectType):
     class Meta:
         model = models.Task
-        exclude = ("task_flows", "work_items", "form")
+        exclude = ("task_flows", "work_items", "form", "type")
         use_connection = False
         interfaces = (Task, relay.Node)
 
@@ -114,7 +138,7 @@ class CompleteTaskFormTask(TaskQuerysetMixin, DjangoObjectType):
 
     class Meta:
         model = models.Task
-        exclude = ("task_flows", "work_items")
+        exclude = ("task_flows", "work_items", "type")
         use_connection = False
         interfaces = (Task, relay.Node)
 
@@ -130,6 +154,7 @@ class Flow(DjangoObjectType):
         model = models.Flow
         interfaces = (relay.Node,)
         connection_class = CountableConnectionBase
+        fields = "__all__"
 
 
 class Workflow(DjangoObjectType):
@@ -155,7 +180,11 @@ class Workflow(DjangoObjectType):
         return self.start_tasks.all()
 
     flows = DjangoFilterConnectionField(
-        Flow, filterset_class=CollectionFilterSetFactory(filters.FlowFilterSet)
+        Flow,
+        filterset_class=CollectionFilterSetFactory(
+            filterset_class=filters.FlowFilterSet,
+            orderset_class=filters.FlowOrderSet,
+        ),
     )
 
     class Meta:
@@ -171,36 +200,68 @@ class WorkItem(DjangoObjectType):
     assigned_users = graphene.List(graphene.String, required=True)
     controlling_groups = graphene.List(graphene.String, required=True)
     addressed_groups = graphene.List(graphene.String, required=True)
+    status = WorkItemStatus(required=True)
+    is_redoable = graphene.Boolean(
+        description=(
+            "This property potentially performs poorly if used in a large set"
+            "of entries, as the evaluation of the redoable jexl configuration"
+            "cannot be performed on the database level. Please use carefully."
+        )
+    )
+
+    resolve_case = suppressable_visibility_resolver()
+    resolve_child_case = suppressable_visibility_resolver()
+    resolve_task = suppressable_visibility_resolver()
+    resolve_document = suppressable_visibility_resolver()
+    resolve_previous_work_item = suppressable_visibility_resolver()
+
+    def resolve_is_redoable(self, *args, **kwargs):
+        return (
+            self.status != models.WorkItem.STATUS_READY
+            and RedoWorkItemLogic.is_work_item_redoable(self)
+        )
 
     class Meta:
         model = models.WorkItem
         interfaces = (relay.Node,)
         connection_class = CountableConnectionBase
+        fields = "__all__"
 
 
 class Case(DjangoObjectType):
     work_items = DjangoFilterConnectionField(
         WorkItem,
         filterset_class=CollectionFilterSetFactory(
-            filters.WorkItemFilterSet, orderset_class=filters.WorkItemOrderSet
+            filterset_class=filters.WorkItemFilterSet,
+            orderset_class=filters.WorkItemOrderSet,
         ),
     )
     family_work_items = DjangoFilterConnectionField(
         WorkItem,
         filterset_class=CollectionFilterSetFactory(
-            filters.WorkItemFilterSet, orderset_class=filters.WorkItemOrderSet
+            filterset_class=filters.WorkItemFilterSet,
+            orderset_class=filters.WorkItemOrderSet,
         ),
     )
     meta = generic.GenericScalar()
+    status = CaseStatus(required=True)
+
+    resolve_document = suppressable_visibility_resolver()
+
+    resolve_parent_work_item = suppressable_visibility_resolver()
+
+    resolve_family = suppressable_visibility_resolver()
+
+    resolve_workflow = suppressable_visibility_resolver()
 
     def resolve_family_work_items(self, info, **args):
         return models.WorkItem.objects.filter(case__family=self.family)
 
     class Meta:
         model = models.Case
-        exclude = ("family",)
         interfaces = (relay.Node,)
         connection_class = CountableConnectionBase
+        fields = "__all__"
 
 
 class SaveWorkflow(UserDefinedPrimaryKeyMixin, Mutation):
@@ -253,13 +314,10 @@ class SaveCompleteTaskFormTask(SaveTask):
         return_field_type = Task
 
 
-class StartCase(Mutation):
-    class Meta:
-        serializer_class = serializers.CaseSerializer
-        model_operations = ["create"]
-
-
 class SaveCase(Mutation):
+    class Input:
+        id = graphene.String()
+
     class Meta:
         serializer_class = serializers.SaveCaseSerializer
         model_operations = ["create", "update"]
@@ -283,6 +341,20 @@ class ResumeCase(Mutation):
         model_operations = ["update"]
 
 
+class ReopenCase(Mutation):
+    class Input:
+        id = graphene.ID(required=True)
+        work_items = graphene.List(
+            graphene.ID,
+            required=True,
+            description="List of work item ids to be readied when the case is reopened",
+        )
+
+    class Meta:
+        serializer_class = serializers.ReopenCaseSerializer
+        model_operations = ["update"]
+
+
 class CompleteWorkItem(Mutation):
     class Meta:
         serializer_class = serializers.CompleteWorkItemSerializer
@@ -341,14 +413,12 @@ class Mutation(object):
     save_complete_workflow_form_task = SaveCompleteWorkflowFormTask().Field()
     save_complete_task_form_task = SaveCompleteTaskFormTask().Field()
 
-    start_case = StartCase().Field()
-
-    start_case.deprecation_reason = "Use SaveCase mutation instead"
-
     save_case = SaveCase().Field()
     cancel_case = CancelCase().Field()
     suspend_case = SuspendCase().Field()
     resume_case = ResumeCase().Field()
+    reopen_case = ReopenCase().Field()
+
     complete_work_item = CompleteWorkItem().Field()
     skip_work_item = SkipWorkItem().Field()
     cancel_work_item = CancelWorkItem().Field()
@@ -363,25 +433,29 @@ class Query(object):
     all_workflows = DjangoFilterConnectionField(
         Workflow,
         filterset_class=CollectionFilterSetFactory(
-            filters.WorkflowFilterSet, orderset_class=filters.WorkflowOrderSet
+            filterset_class=filters.WorkflowFilterSet,
+            orderset_class=filters.WorkflowOrderSet,
         ),
     )
-    all_tasks = DjangoFilterSetConnectionField(
+    all_tasks = DjangoFilterInterfaceConnectionField(
         TaskConnection,
         filterset_class=CollectionFilterSetFactory(
-            filters.TaskFilterSet, orderset_class=filters.TaskOrderSet
+            filterset_class=filters.TaskFilterSet,
+            orderset_class=filters.TaskOrderSet,
         ),
     )
     all_cases = DjangoFilterConnectionField(
         Case,
         filterset_class=CollectionFilterSetFactory(
-            filters.CaseFilterSet, orderset_class=filters.CaseOrderSet
+            filterset_class=filters.CaseFilterSet,
+            orderset_class=filters.CaseOrderSet,
         ),
     )
 
     all_work_items = DjangoFilterConnectionField(
         WorkItem,
         filterset_class=CollectionFilterSetFactory(
-            filters.WorkItemFilterSet, orderset_class=filters.WorkItemOrderSet
+            filterset_class=filters.WorkItemFilterSet,
+            orderset_class=filters.WorkItemOrderSet,
         ),
     )
