@@ -36,6 +36,20 @@ class GroupJexlField(serializers.JexlField):
         super().__init__(GroupJexl(), **kwargs)
 
 
+class CaseStatusField(serializers.CalumaChoiceField):
+    # Required for the type system, case status is
+    # never part of an input
+    def __init__(self, **kwargs):  # pragma: no cover
+        super().__init__([s for s, _ in models.Case.STATUS_CHOICE_TUPLE], **kwargs)
+
+
+class WorkItemStatusField(serializers.CalumaChoiceField):
+    # Required for the type system, workitem status is
+    # never part of an input
+    def __init__(self, **kwargs):  # pragma: no cover
+        super().__init__([s for s, _ in models.WorkItem.STATUS_CHOICE_TUPLE], **kwargs)
+
+
 class SaveWorkflowSerializer(serializers.ModelSerializer):
     class Meta:
         model = models.Workflow
@@ -60,18 +74,22 @@ class AddWorkflowFlowSerializer(serializers.ModelSerializer):
     next = FlowJexlField(required=True)
     redoable = FlowJexlField(required=False, write_only=True)
 
-    def validate_next(self, value):
-        jexl = FlowJexl()
-        tasks = set(jexl.extract_tasks(value))
+    def has_tasks_in_jexl(self, expression):
+        if not expression:
+            return False
 
-        if not tasks:
+        return len(set(FlowJexl().extract_tasks(expression))) > 0
+
+    def validate(self, data):
+        next_has_tasks = self.has_tasks_in_jexl(data.get("next"))
+        redoable_has_tasks = self.has_tasks_in_jexl(data.get("redoable"))
+
+        if not next_has_tasks and not redoable_has_tasks:
             raise exceptions.ValidationError(
-                f"jexl `{value}` does not contain any tasks as return value"
+                "Either `next` or `redoable` must contain tasks"
             )
-        return value
 
-    def validate_redoable(self, value):
-        return self.validate_next(value)
+        return super().validate(data)
 
     @transaction.atomic
     def update(self, instance, validated_data):
@@ -124,6 +142,14 @@ class SaveTaskSerializer(serializers.ModelSerializer):
         help_text=models.Task._meta.get_field("control_groups").help_text,
     )
 
+    def validate(self, data):
+        if data.get("continue_async") and not data.get("is_multiple_instance"):
+            raise ValidationError(
+                "Only multiple instance tasks can continue asynchronously"
+            )
+
+        return data
+
     class Meta:
         model = models.Task
         fields = [
@@ -136,6 +162,7 @@ class SaveTaskSerializer(serializers.ModelSerializer):
             "is_archived",
             "lead_time",
             "is_multiple_instance",
+            "continue_async",
         ]
 
 
@@ -170,7 +197,7 @@ class SaveCompleteTaskFormTaskSerializer(SaveTaskSerializer):
         fields = SaveTaskSerializer.Meta.fields + ["form"]
 
 
-class CaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
+class SaveCaseSerializer(ContextModelSerializer):
     workflow = serializers.GlobalIDPrimaryKeyRelatedField(
         queryset=models.Workflow.objects.prefetch_related("start_tasks")
     )
@@ -193,44 +220,32 @@ class CaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
     def create(self, validated_data):
         user = self.context["request"].user
 
-        validated_data = domain_logic.StartCaseLogic.pre_start(validated_data, user)
+        # Remove parent_work_item from validated_data before case creation.
+        #
+        # Django has never saved reverse one-to-one relation and starting with
+        # version 5.2 raises an exception when passing one as a param to the
+        # create method.
+        #
+        # See https://code.djangoproject.com/ticket/34586
+        parent_work_item = validated_data.pop("parent_work_item", None)
+
+        validated_data = domain_logic.StartCaseLogic.pre_start(
+            validated_data, user, parent_work_item
+        )
 
         case = super().create(validated_data)
 
         return domain_logic.StartCaseLogic.post_start(
-            case, user, validated_data.get("parent_work_item"), self.context_data
+            case, user, parent_work_item, self.context_data
         )
 
     class Meta:
         model = models.Case
-        fields = ["workflow", "meta", "parent_work_item", "form", "context"]
-
-
-class SaveCaseSerializer(CaseSerializer):
-    @transaction.atomic
-    def create(self, validated_data):
-        self.send_event(
-            events.pre_create_case, case=None, validated_data=validated_data
-        )
-        instance = super().create(validated_data)
-        self.send_event(events.post_create_case, case=instance)
-        return instance
-
-    @transaction.atomic
-    def update(self, instance, validated_data):
-        self.send_event(
-            events.pre_create_case, case=instance, validated_data=validated_data
-        )
-        instance = super().update(instance, validated_data)
-        self.send_event(events.post_create_case, case=instance)
-        return instance
-
-    class Meta(CaseSerializer.Meta):
         fields = ["id", "workflow", "meta", "parent_work_item", "form", "context"]
         extra_kwargs = {"id": {"read_only": False, "required": False}}
 
 
-class CancelCaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
+class CancelCaseSerializer(ContextModelSerializer):
     id = serializers.GlobalIDField()
 
     class Meta:
@@ -261,7 +276,7 @@ class CancelCaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
         return case
 
 
-class SuspendCaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
+class SuspendCaseSerializer(ContextModelSerializer):
     id = serializers.GlobalIDField()
 
     class Meta:
@@ -292,7 +307,7 @@ class SuspendCaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
         return case
 
 
-class ResumeCaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
+class ResumeCaseSerializer(ContextModelSerializer):
     id = serializers.GlobalIDField()
 
     class Meta:
@@ -323,13 +338,43 @@ class ResumeCaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
         return case
 
 
+class ReopenCaseSerializer(ContextModelSerializer):
+    id = serializers.GlobalIDField()
+    work_items = ListField(child=serializers.GlobalIDField(), required=True)
+
+    class Meta:
+        model = models.Case
+        fields = ["id", "work_items", "context"]
+
+    def validate(self, data):
+        domain_logic.ReopenCaseLogic.validate_for_reopen(
+            self.instance, self.instance.work_items.filter(pk__in=data["work_items"])
+        )
+
+        return super().validate(data)
+
+    @transaction.atomic
+    def update(self, case, validated_data):
+        user = self.context["request"].user
+
+        work_items = case.work_items.filter(pk__in=validated_data["work_items"])
+
+        domain_logic.ReopenCaseLogic.pre_reopen(case, work_items, user, self.context)
+
+        domain_logic.ReopenCaseLogic.do_reopen(case, work_items)
+
+        domain_logic.ReopenCaseLogic.post_reopen(case, work_items, user, self.context)
+
+        return case
+
+
 class CompleteWorkItemSerializer(ContextModelSerializer):
     id = serializers.GlobalIDField()
 
     def validate(self, data):
         try:
             domain_logic.CompleteWorkItemLogic.validate_for_complete(
-                self.instance, self.context["request"].user
+                self.instance, self.context["request"].user, context=data.get("context")
             )
         except ValidationError as e:
             raise exceptions.ValidationError(str(e))
@@ -418,7 +463,7 @@ class CancelWorkItemSerializer(ContextModelSerializer):
         fields = ["id", "context"]
 
 
-class SaveWorkItemSerializer(SendEventSerializerMixin, ContextModelSerializer):
+class SaveWorkItemSerializer(ContextModelSerializer):
     work_item = serializers.GlobalIDField(source="id")
     name = CharField(
         required=False,
@@ -433,17 +478,6 @@ class SaveWorkItemSerializer(SendEventSerializerMixin, ContextModelSerializer):
         help_text=models.WorkItem._meta.get_field("description").help_text,
     )
 
-    @transaction.atomic
-    def update(self, instance, validated_data):
-        self.send_event(
-            events.pre_create_work_item,
-            work_item=instance,
-            validated_data=validated_data,
-        )
-        instance = super().update(instance, validated_data)
-        self.send_event(events.post_create_work_item, work_item=instance)
-        return instance
-
     class Meta:
         model = models.WorkItem
         fields = [
@@ -620,7 +654,9 @@ class WorkItemRedoTaskSerializer(ContextModelSerializer):
         )
 
         domain_logic.RedoWorkItemLogic.set_succeeding_work_item_status_redo(work_item)
+
         work_item = super().update(work_item, validated_data)
+
         work_item = domain_logic.RedoWorkItemLogic.post_redo(
             work_item, user, self.context_data
         )
