@@ -90,6 +90,40 @@ def create_work_items(tasks, case, user, prev_work_item=None, context: dict = No
             work_item_groups = [addressed_groups]
 
         for groups in work_item_groups:
+            if (
+                not task.is_multiple_instance
+                and models.WorkItem.objects.filter(
+                    addressed_groups=groups,
+                    controlling_groups=controlling_groups,
+                    task_id=task.pk,
+                    case=case,
+                    status__in=[
+                        models.WorkItem.STATUS_READY,
+                        models.WorkItem.STATUS_SUSPENDED,
+                    ],
+                ).exists()
+            ):
+                # work item already exists, do not create a new one
+                continue
+
+            work_items_to_redo = models.WorkItem.objects.filter(
+                addressed_groups=groups,
+                controlling_groups=controlling_groups,
+                task_id=task.pk,
+                case=case,
+                status=models.WorkItem.STATUS_REDO,
+            )
+
+            if work_items_to_redo.exists():
+                for work_item in work_items_to_redo:
+                    work_item.deadline = task.calculate_deadline()
+                    work_item.modified_by_user = user.username
+                    work_item.modified_by_group = user.username
+                    work_item.status = models.WorkItem.STATUS_READY
+                    work_item.save()
+                    work_items.append(work_item)
+                continue
+
             work_items.append(
                 models.WorkItem.objects.create(
                     addressed_groups=groups,
