@@ -1,18 +1,26 @@
+import contextlib
 import datetime
 import functools
 import inspect
+import itertools
+import sys
+from collections import defaultdict
 
 import pytest
 import urllib3
 from django.apps import apps
 from django.core.cache import cache
+from django.db import connection
+from django.db.migrations.executor import MigrationExecutor
 from factory import Faker
 from factory.base import FactoryMetaClass
 from graphene import ResolveInfo
 from minio import Minio
 from minio.datatypes import Object as MinioStatObject
 from pytest_factoryboy import register
+from pytest_factoryboy.fixture import Box
 
+from .caluma_analytics import factories as analytics_factories
 from .caluma_core.faker import MultilangProvider
 from .caluma_core.models import HistoricalRecords
 from .caluma_form import factories as form_factories
@@ -25,14 +33,20 @@ Faker.add_provider(MultilangProvider)
 
 
 def register_module(module):
+    # We need to pass the locals of this file to the register method to make
+    # sure they are injected on the conftest locals instead of the default
+    # locals which would be the locals of this function
+    conftest_locals = Box(sys._getframe(1).f_locals)
+
     for _, obj in inspect.getmembers(module):
         if isinstance(obj, FactoryMetaClass) and not obj._meta.abstract:
-            register(obj)
+            register(obj, _caller_locals=conftest_locals)
 
 
 register_module(form_factories)
 register_module(logging_factories)
 register_module(workflow_factories)
+register_module(analytics_factories)
 
 
 @pytest.fixture(scope="function", autouse=True)
@@ -69,17 +83,21 @@ def anonymous_request(rf):
 @pytest.fixture
 def info(anonymous_request):
     """Mock for GraphQL resolve info embedding django request as context."""
+
     return ResolveInfo(
         None,
         None,
         None,
         None,
-        schema=None,
-        fragments=None,
-        root_value=None,
-        operation=None,
-        variable_values=None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
         context=anonymous_request,
+        # TODO: see if this has an influence on execution, might need parametrization
+        is_awaitable=lambda x: False,
     )
 
 
@@ -91,12 +109,15 @@ def admin_info(admin_request):
         None,
         None,
         None,
-        schema=None,
-        fragments=None,
-        root_value=None,
-        operation=None,
-        variable_values=None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
         context=admin_request,
+        # TODO: see if this has an influence on execution, might need parametrization
+        is_awaitable=lambda x: False,
     )
 
 
@@ -230,6 +251,10 @@ def form_and_document(
     Optionally add a table question (and a row in the document),
     and a subform for added complexity.
 
+    You can call the factory multiple times, in which case the
+    created forms will be the same, but new document structures
+    will be returned each time.
+
     The slugs are named as follows:
 
     * form: top_form
@@ -241,70 +266,101 @@ def form_and_document(
            * sub_form: sub_form
                * question: sub_question
     """
+    sorter = itertools.count(9999, -1)
 
-    def factory(use_table=False, use_subform=False):
-        form = form_factory(slug="top_form", meta={"is-top-form": True, "level": 0})
+    def fallback_factory(factory, **kwargs):
+        existing = factory._meta.model.objects.filter(**kwargs).first()
+        if existing:
+            return existing
+        if factory is form_question_factory:
+            # Make form structure / sorting deterministic
+            kwargs["sort"] = next(sorter)
+        return factory(**kwargs)
+
+    def factory(use_table=False, use_subform=False, table_row_count=1):
+        form = fallback_factory(
+            form_factory, slug="top_form", meta={"is-top-form": True, "level": 0}
+        )
         document = document_factory(form=form)
 
         questions = {}
         answers = {}
 
-        questions["top_question"] = question_factory(
+        questions["top_question"] = fallback_factory(
+            question_factory,
             slug="top_question",
             type="text",
             is_required="true",
             is_hidden="false",
         )
 
-        form_question_factory(form=form, question=questions["top_question"])
+        fallback_factory(
+            form_question_factory, form=form, question=questions["top_question"]
+        )
         answers["top_question"] = answer_factory(
             document=document, question=questions["top_question"]
         )
 
         if use_table:
-            row_form = form_factory(slug="row_form")
-            questions["table"] = question_factory(
+            row_form = fallback_factory(form_factory, slug="row_form")
+            questions["table"] = fallback_factory(
+                question_factory,
                 type="table",
                 slug="table",
                 row_form=row_form,
                 is_required="true",
                 is_hidden="false",
             )
-            form_question_factory(form=form, question=questions["table"])
-            questions["column"] = question_factory(
-                type="float", slug="column", is_required="true", is_hidden="false"
+            fallback_factory(
+                form_question_factory, form=form, question=questions["table"]
+            )
+            questions["column"] = fallback_factory(
+                question_factory,
+                type="float",
+                slug="column",
+                is_required="true",
+                is_hidden="false",
+            )
+            fallback_factory(
+                form_question_factory, form=row_form, question=questions["column"]
             )
-            form_question_factory(form=row_form, question=questions["column"])
 
             answers["table"] = answer_factory(
                 document=document, question=questions["table"]
             )
 
-            row_doc = document_factory(form=row_form, family=document)
-            answers["column"] = answer_factory(
-                document=row_doc, question=questions["column"]
-            )
-            answers["table"].documents.add(row_doc)
+            for _ in range(table_row_count):
+                row_doc = document_factory(form=row_form, family=document)
+                answers["column"] = answer_factory(
+                    document=row_doc, question=questions["column"]
+                )
+                answers["table"].documents.add(row_doc)
 
         if use_subform:
-            sub_form = form_factory(
-                slug="sub_form", meta={"is-top-form": False, "level": 1}
+            sub_form = fallback_factory(
+                form_factory, slug="sub_form", meta={"is-top-form": False, "level": 1}
             )
-            questions["form"] = question_factory(
+            questions["form"] = fallback_factory(
+                question_factory,
                 type="form",
                 slug="form",
                 sub_form=sub_form,
                 is_required="true",
                 is_hidden="false",
             )
-            form_question_factory(form=form, question=questions["form"])
-            questions["sub_question"] = question_factory(
+            fallback_factory(
+                form_question_factory, form=form, question=questions["form"]
+            )
+            questions["sub_question"] = fallback_factory(
+                question_factory,
                 slug="sub_question",
-                type="text",
+                type="integer",
                 is_required="true",
                 is_hidden="false",
             )
-            form_question_factory(form=sub_form, question=questions["sub_question"])
+            fallback_factory(
+                form_question_factory, form=sub_form, question=questions["sub_question"]
+            )
 
             answers["sub_question"] = answer_factory(
                 document=document, question=questions["sub_question"]
@@ -335,3 +391,57 @@ def sorted_snapshot(snapshot):
         return snapshot(matcher=_sorted(*args))
 
     return custom_snapshot
+
+
+@pytest.fixture
+def set_date(freezer):
+    """Run a block of code at a certain date (context manager).
+
+    Uses freezegun, but allows you to override the frozen date.
+
+    Provides a context manager, where everything within the "with"
+    block is run at the given date:
+
+    >>> @pytest.mark.freeze_time('2022-02-22')
+    >>> def test_foo(set_date):
+            assert datetime.now() == datetime(2022,2,22)
+
+            with set_date('2022-01-01'):
+                assert datetime.now() == datetime(2022,1,1)
+
+            assert datetime.now() == datetime(2022,2,22)
+    """
+
+    @contextlib.contextmanager
+    def make_context(date):
+        old_now = datetime.datetime.now()
+        freezer.move_to(date)
+        yield
+        freezer.move_to(old_now)
+
+    return make_context
+
+
+@pytest.fixture
+def post_migrate_to_current_state(transactional_db):
+    """
+    Apply all current migrations after test has run.
+
+    In migration-tests, the `transactional_db` fixture seems to fail to apply the
+    newest migrations for all apps, leading to flaky tests. This fixture makes sure,
+    that all apps are migrated to their most current state.
+    """
+    try:
+        yield
+    finally:
+        executor = MigrationExecutor(connection)
+        migrations_dict = defaultdict(list)
+        for app, migration in executor.loader.disk_migrations.keys():
+            migrations_dict[app].append(migration)
+
+        migrations_list = [
+            (app, max(migrations)) for app, migrations in migrations_dict.items()
+        ]
+        executor.loader.build_graph()
+        executor.migrate(migrations_list)
+        executor.loader.build_graph()
