@@ -5,6 +5,7 @@ from graphene_django.converter import convert_django_field, convert_field_to_str
 from graphene_django.debug import DjangoDebug
 from localized_fields.fields import LocalizedField
 
+from .caluma_analytics import schema as analytics_schema
 from .caluma_data_source import schema as data_source_schema
 from .caluma_form import (
     historical_schema as form_historical_schema,
@@ -18,7 +19,12 @@ convert_django_field.register(LocalizedField, convert_field_to_string)
 # exporting it as `Mutation`, which can be confusing
 _mutation = type(
     "Mutation",
-    (form_schema.Mutation, workflow_schema.Mutation, graphene.ObjectType),
+    (
+        form_schema.Mutation,
+        workflow_schema.Mutation,
+        analytics_schema.Mutation,
+        graphene.ObjectType,
+    ),
     {},
 )
 
@@ -26,6 +32,7 @@ query_inherit_from = [
     form_schema.Query,
     workflow_schema.Query,
     data_source_schema.Query,
+    analytics_schema.Query,
     graphene.ObjectType,
 ]
 
@@ -41,7 +48,7 @@ types = [
     form_schema.DateQuestion,
     form_schema.TableQuestion,
     form_schema.FormQuestion,
-    form_schema.FileQuestion,
+    form_schema.FilesQuestion,
     form_schema.StaticQuestion,
     form_schema.StringAnswer,
     form_schema.ListAnswer,
@@ -49,12 +56,14 @@ types = [
     form_schema.FloatAnswer,
     form_schema.DateAnswer,
     form_schema.TableAnswer,
-    form_schema.FileAnswer,
+    form_schema.FilesAnswer,
     form_schema.CalculatedFloatQuestion,
     form_schema.ActionButtonQuestion,
     workflow_schema.SimpleTask,
     workflow_schema.CompleteWorkflowFormTask,
     workflow_schema.CompleteTaskFormTask,
+    analytics_schema.AnalyticsTable,
+    analytics_schema.AnalyticsField,
 ]
 
 historical_types = [
@@ -64,7 +73,7 @@ historical_types = [
     form_historical_schema.HistoricalFloatAnswer,
     form_historical_schema.HistoricalDateAnswer,
     form_historical_schema.HistoricalTableAnswer,
-    form_historical_schema.HistoricalFileAnswer,
+    form_historical_schema.HistoricalFilesAnswer,
 ]
 
 if settings.ENABLE_HISTORICAL_API:
