@@ -1,11 +1,11 @@
 """
-This settings module only contains caluma specific settings.
+Caluma specific settings.
 
-It's imported by the main caluma settings and is intended to also be used by third party
+This settings module only contains caluma-internal settings. It's imported
+by the main caluma settings and is intended to also be used by third party
 applications integrating Caluma.
 """
 
-
 import os
 
 import environ
@@ -52,20 +52,29 @@ GRAPHENE = {
     "RELAY_CONNECTION_MAX_LIMIT": None,
 }
 
+# If you set DISABLE_INTROSPECTION to True, any GQL client will not be able to
+# query the types and connections, making crafting queries much harder (but not
+# impossible, and this is an Open Source product, so they can still go check
+# the source).
+DISABLE_INTROSPECTION = env.bool("DISABLE_INTROSPECTION", default=default(False, True))
+
+# DOS protection: Limit query depth to a given level. Default is 0, which means
+# it is disabled
+QUERY_DEPTH_LIMIT = env.int("QUERY_DEPTH_LIMIT", default=0)
+
 # OpenID connect
 
 OIDC_USERINFO_ENDPOINT = env.str("OIDC_USERINFO_ENDPOINT", default=None)
 OIDC_VERIFY_SSL = env.bool("OIDC_VERIFY_SSL", default=True)
 OIDC_GROUPS_CLAIM = env.str("OIDC_GROUPS_CLAIM", default="caluma_groups")
 OIDC_USERNAME_CLAIM = env.str("OIDC_USERNAME_CLAIM", default="sub")
+# Claim to use for the "client" claim. Should normally be left to the default value
+OIDC_CLIENT_CLAIM = env.str("OIDC_CLIENT_CLAIM", default="client_id")
+
 OIDC_BEARER_TOKEN_REVALIDATION_TIME = env.int(
     "OIDC_BEARER_TOKEN_REVALIDATION_TIME", default=0
 )
 
-OIDC_INTROSPECT_ENDPOINT = env.str("OIDC_INTROSPECT_ENDPOINT", default=None)
-OIDC_INTROSPECT_CLIENT_ID = env.str("OIDC_INTROSPECT_CLIENT_ID", default=None)
-OIDC_INTROSPECT_CLIENT_SECRET = env.str("OIDC_INTROSPECT_CLIENT_SECRET", default=None)
-
 CALUMA_OIDC_USER_FACTORY = env.str(
     "CALUMA_OIDC_USER_FACTORY", default="caluma.caluma_user.models.OIDCUser"
 )
@@ -100,7 +109,8 @@ SIMPLE_HISTORY_HISTORY_ID_USE_UUID = True
 ENABLE_HISTORICAL_API = env.bool("ENABLE_HISTORICAL_API", default=False)
 
 # Configure the fields you intend to use in the "meta" fields. This will
-# provide corresponding constants in the ordreBy filter.
+# provide corresponding constants in the ordreBy filter, as well as allow
+# you to use those fields in the analytics module.
 META_FIELDS = env.list("META_FIELDS", default=[])
 
 # enable caluma healthz endpoint
