@@ -25,6 +25,7 @@ INSTALLED_APPS = [
     "caluma.caluma_core.apps.DefaultConfig",
     "caluma.caluma_user.apps.DefaultConfig",
     "caluma.caluma_form.apps.DefaultConfig",
+    "caluma.caluma_analytics.apps.DefaultConfig",
     "caluma.caluma_workflow.apps.DefaultConfig",
     "caluma.caluma_data_source.apps.DefaultConfig",
     "caluma.caluma_logging.apps.DefaultConfig",
@@ -32,7 +33,14 @@ INSTALLED_APPS = [
 ]
 
 if DEBUG:
-    INSTALLED_APPS.append("django_extensions")
+    try:
+        __import__("django_extensions")
+        INSTALLED_APPS.append("django_extensions")
+    except ImportError:  # pragma: no cover
+        # Nothing bad, just won't have django-extensions
+        # niceties installed (Most likely Caluma was built
+        # without dev dependencies)
+        pass
 
 MIDDLEWARE = [
     "django.middleware.security.SecurityMiddleware",
@@ -74,6 +82,8 @@ DATABASES = {
     }
 }
 
+DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
+
 # Cache
 # https://docs.djangoproject.com/en/1.11/ref/settings/#caches
 
@@ -101,7 +111,6 @@ LANGUAGES = (
 
 TIME_ZONE = env.str("TIME_ZONE", "UTC")
 USE_I18N = True
-USE_L10N = True
 USE_TZ = True
 
 
@@ -190,7 +199,6 @@ WATCHMAN_CHECKS = env.list(
     default=(
         "caluma.caluma_core.health_checks.check_migrations",
         "caluma.caluma_core.health_checks.check_media_storage_service",
-        "caluma.caluma_core.health_checks.check_models",
         "watchman.checks.caches",
         "watchman.checks.databases",
     ),
